var util = require('util');
var request = require('request');
var uuid = require('node-uuid');
var java = require('java');
var logger = require('logger')(__filename);
var bodyTemplate = "<speak version=\"1.0\" xmlns=\"http://www.w3.org/2001/10/synthesis\" xmlns:mstts=\"http://www.w3.org/2001/mstts\" xmlns:emo=\"http://www.w3.org/2009/10/emotionml\" xml:lang=\"%s\">%s<emo:emotion><emo:category name=\"CALM\" value=\"1.0\"/>%s</emo:emotion></voice></speak>";

function getWave(options, cb) {
  options = options || {};
  options.contentType = options.contentType || 'application/ssml+xml';
  logger.log('calling speech with format', options.outputFormat);
  return java.getAuthenticationTokens(function(err, tokens) {
        if (err) {
          logger.error('failed to get authentication tokens');
          return cb(err);
        }

        var aisToken = tokens.aisToken;
        var rpsToken = tokens.rpsToken;

        var response = request.post(
          'https://speech.platform.bing.com/synthesize/android/synth?outputformat='+options.outputFormat, {
              body: options.body,
              headers: {
                'X-MICROSOFT-OutputFormat': options.outputFormat,
                'X-FD-ClientID': 'TtsEndToEndTestClient',
                'X-FD-ImpressionGUID': uuid.v1(),
                'Content-Type': options.contentType,
                'Cookie': '_SS=Moderate;',
                'X-Search-RPSToken' : rpsToken,
                'X-AIS-AuthToken': aisToken
              }
          });
        return cb(null, response);
  });
}

function speak(text, outputFormat, appLanguage, cb) {
  var start = new Date().getTime();
  
  var voiceName = "<voice name=\"Microsoft Server Speech Text to Speech Voice (en-US, EvaRUS)\">";
  if (appLanguage.toLowerCase() === "zh-cn") {
    voiceName = "<voice name=\"Microsoft Server Speech Text to Speech Voice (zh-CN, HongyuRUS)\">";
  }
  var body = util.format(bodyTemplate, appLanguage, voiceName, text);
    var options = {
      body: body,
      outputFormat: outputFormat

    };

    logger.log('get wave with body', body);
    getWave(options, function(err, ttsResponse) {
      if (err) {
          logger.error('failed to get wave', err);
          return cb(err);
        }

        ttsResponse.on('response', function(response){
            if(response.statusCode !== 200) {
                var msg = util.format('error getting cortana tts for text: %s, status code: %d', text, response.statusCode);
                return cb(new Error(msg));
            }

        var end = new Date().getTime();
        var time = end - start;
        console.log('speak execution time: ' + time);
            return cb(null, response);
        });

        ttsResponse.on('error', function(err){
            var msg = util.format('error getting cortana tts for text: %s, error: %s', text, err);
            return cb(new Error(msg));
        });
    });
}

module.exports = {
    requestCortanaSpeak: speak
}
