// requiring java is the first thing we want to do as it initializes the console.log bridge from js to adb logs.
// only after requiring java we'll be able to see console.log in adb. any console.log above this line will not be seen.
var java = require('java');
var http = require('http');
var middler = require('middler');
var connectmw = require('connect-mw');
var request = require('request');
var logger = require('logger')(__filename);

logger.log('running node version', process.version, __dirname, __filename);

// this code is called from the native ndknode.cpp file in case that there
// is an uncaught exception in a runnable running on the container/node's thread
process.crash = function (err) {
  logger.warn('crashing process', err);
  process.nextTick(function () {
    throw err;
  });
}

process.on('uncaughtException', function (err) {
  logger.error('uncaughtException occurred', err, err.stack);

  // rethrowing error so that app is crashed and
  // error message is transferred to the Java application layer
  throw err;
});

// set up the proxy for the request module methods in case there is one set for android
if (process.platform == 'android') {
  var proxyAddress = java.getConfiguration().getStringConfig("proxyAddress");
  var proxyPort = java.getConfiguration().getIntegerConfig("proxyPort");

  if (proxyAddress && proxyPort && proxyPort !== -1) {
    var proxy = "http://" + proxyAddress + ":" + proxyPort;
    logger.log('setting proxy:', proxy);
    process.env['NODE_TLS_REJECT_UNAUTHORIZED'] = '0';

    var methodsToOverride = ["post", "get", "delete", "put"];

    for (var i = 0; i< methodsToOverride.length; i++) {
      var methodToOverride = methodsToOverride[i];
      var original = request[methodToOverride];
      (function(methodToOverride, original) {
        request[methodToOverride] = function() {
          // request's method generally accepts the following arguments:
          // (uri, options, cb )
          // where uri is optional and can come as part of the options object
          // options may also be optional if only uri is specified
          // here, we first check if the second argument is an object,
          // if it is, we set the proxy object on it, as request expects
          // if the 2nd arguemnt isn't an object, but the first one is, 
          // we set the proxy field of the first argument.
          
          if (typeof arguments[1] == "object") {
            arguments[1].proxy = arguments[1].proxy || proxy;
          }
          else if (typeof arguments[0] == "object") {
            arguments[0].proxy = arguments[0].proxy || proxy;
          }

          return original.apply(this, arguments);
        }
      })(methodToOverride, original);
    }
  }
}

logger.log('process.argv:', process.argv);
logger.log('process.execArgv:', process.execArgv);

/* we currently don't use any argument from command line
process.argv.forEach(function(arg) {
  var argElements = arg.split("=");
  if (argElements.length == 2)
    process.env[argElements[0]] = argElements[1] || process.env[argElements[0]];
    logger.log('new command line env variable', argElements);
});
*/

var NODE_AUTH_COOKIE_NAME = 'X-Dss-Auth';
function processDssAuthCookie(request) {
  var rc = request.headers.cookie;
  
  if (!rc) {
    return false;
  }

  // work around a client bug where cookies set by adding a Cookies header and cookies set by adding a cookie to the
  // cookie manager end up comma separated.... 'one=two; three=four;, five=size'.
  rc = rc.split(',').join('');

  var authToken;
  var authCookieIndex;
  var cookies = rc.split(';');
  cookies.some(function (cookie, index) {
    var parts = cookie.split('=');
    var name = parts[0].trim();

    if (name === NODE_AUTH_COOKIE_NAME) {
      authToken = parts[1].trim();
      authCookieIndex = index;
      return true;
    }

    return false;
  });
  
  if (authToken) {
    // Remove the dss auth token so it can't be forwarded to the network.
    cookies.splice(authCookieIndex, 1);
    request.headers.cookie = cookies.join(';');
    logger.log('found dss auth cookie in request');
  }
  else {
    logger.error('cannot find dss auth cookie in request');
  }
  
  return authToken;
}

// Generate the auth id for this instance.
var authCookieUuid = require('node-uuid').v4();

var app = middler()
.first(function (req, res, next) {
  // Check that the required auth cookie is sent.  This is passed back in the Node Ready Event.
  // It is expected that only authorized clients can access the header value through
  // a secure service endpoint.
  var authToken = processDssAuthCookie(req);  

  if (!authToken || authToken !== authCookieUuid) {
    logger.log(req.method, req.url, 'not authenticated');
  
    res.writeHead(401);
    return res.end('Unauthorized.');
  }

  logger.log(req.method, req.url, 'authenticated');
  return next();
})
.last(function (req, res) {
  res.writeHead(404);
  return res.end(req.method + ' ' + req.url + ' not found');
})
.add(connectmw.json());

var api = require('./api.js');
app.add(api);

var services = require('./services');
app.add(services());

var proactive = require('./proactive');
app.add(proactive);

// start servers
var server = http.createServer();
middler(server).add(app);

var port = 0; // 0- dynamically allocated port
server.listen(port, function(err) {
  if (err) {
    return logger.error('Failed to listen', err);
  }
  port = server.address().port;

  logger.log('notifying cortana app that we\'re ready to get connections...');
  java.notifyNodeReady(port, authCookieUuid);

  logger.log('start listening to Java events');
  java.eventsManager.start();

  var n=1;
  setInterval(function() {
    logger.log('node is alive: ', n++);
  }, 20000);

  logger.log('listening on port', port);
});