// ----------------------------------------------------------------------------------------------------------------------
//   Copyright 2014 (c) Microsoft Corporation.
//   Summary: Defines well known Entities.
// ----------------------------------------------------------------------------------------------------------------------
(function () {
    "use strict"; // whole file
    
    var enums = {
        Unknown                                  : "<unknown>",
        Type                                     : "<http://www.w3.org/1999/02/22-rdf-syntax-ns#type>",
        Statement                                : "<http://www.w3.org/1999/02/22-rdf-syntax-ns#Statement>",
        Subject                                  : "<http://www.w3.org/1999/02/22-rdf-syntax-ns#subject>",
        Predicate                                : "<http://www.w3.org/1999/02/22-rdf-syntax-ns#predicate>",
        Object                                   : "<http://www.w3.org/1999/02/22-rdf-syntax-ns#object>",
        List                                     : "<http://www.w3.org/1999/02/22-rdf-syntax-ns#List>",
        First                                    : "<http://www.w3.org/1999/02/22-rdf-syntax-ns#first>",
        Rest                                     : "<http://www.w3.org/1999/02/22-rdf-syntax-ns#rest>",
        Nil                                      : "<http://www.w3.org/1999/02/22-rdf-syntax-ns#nil>",
        Sparql                                   : "<http://www.w3.org/TR/rdf-sparql-query/>",
        Term                                     : "<http://schema.org/Term>",
        Graph                                    : "<http://schema.org/Graph>",
        Boolean                                  : "<http://schema.org/Boolean>",
        Double                                   : "<http://schema.org/Float>",
        Integer                                  : "<http://schema.org/Integer>",
        Date                                     : "<http://schema.org/Date>",
        Duration                                 : "<http://schema.org/Duration>",
        Text                                     : "<http://schema.org/Text>",
        Geometry                                 : "<http://www.opengis.net/ont/geosparql#wktLiteral>",
        Variable                                 : "<http://schema.org/Variable>",
        Symbol                                   : "<http://schema.org/Symbol>",
        Person                                   : "<http://schema.org/Person>",
        PreferencesType                          : "<http://schema.org/Thing/Preferences>",
        LocalScoutPreferences                    : "<http://schema.org/Thing/LocalScoutPreferences>",
        LocalScoutCategoryGroup                  : "<http://schema.org/Thing/LocalScoutCategoryGroup>",
        PoiType                                  : "<http://schema.org/Enumeration/PoiType>",
        PoiCategory                              : "<http://schema.org/Enumeration/PoiCategory>",
        FavoritesType                            : "<http://schema.org/Thing/Favorites>",
        StoreLocal                               : "<http://schema.org/Intangible/DataStore/LocalDataStore>",
        StoreLive                                : "<http://schema.org/Intangible/DataStore/LiveDataStore>",
        StoreSkyDrive                            : "<http://schema.org/Intangible/DataStore/SkydriveDataStore>",
        BingStore                                : "<http://schema.org/Intangible/DataStore/BingDataStore>",
        Photo                                    : "<http://schema.org/ImageObject>",
        Place                                    : "<http://schema.org/Place>",
        GeoAnnotationCollection                  : "<http://schema.org/Intangible/GeoAnnotationCollection>",
        Route                                    : "<http://schema.org/Route>",
        PostalAddress                            : "<http://schema.org/PostalAddress>",
        Folder                                   : "<http://schema.org/Intangible/DataStore/Folder>",
        TrafficAlerts                            : "<http://schema.org/Intangible/TrafficAlerts>",
        Typemetadata                             : "<http://platform.bing.com/maps/type.type>",
        Propertymetadata                         : "<http://platform.bing.com/maps/type.property>",
        Categorymetadata                         : "<http://platform.bing.com/maps/type.Category>",
        Url                                      : "<http://schema.org/url>",
        Name                                     : "<http://schema.org/name>",
        Description                              : "<http://schema.org/description>",
        DateTaken                                : "<http://schema.org/dateTaken>",
        DateCreated                              : "<http://schema.org/dateCreated>",
        DateModified                             : "<http://schema.org/dateModified>",
        DateAccessed                             : "<http://platform.bing.com/dateAccessed>",
        Caption                                  : "<http://schema.org/caption>",
        Height                                   : "<http://schema.org/height>",
        Width                                    : "<http://schema.org/width>",
        DateExecuted                             : "<http://schema.org/dateExecuted>",
        DateDeleted                              : "<http://schema.org/dateDeleted>",
        Version                                  : "<http://platform.bing.com/maps/Version>",
        ImageUri                                 : "<http://schema.org/image>",
        Store                                    : "<http://schema.org/dataStore>",
        Preferences                              : "<http://platform.bing.com/maps/person.preferences>",
        PersonStore                              : "<http://platform.bing.com/maps/person.store>",
        LocalScout                               : "<http://platform.bing.com/maps/preferences.local_scout>",
        Favorites                                : "<http://platform.bing.com/maps/preferences.favorites>",
        LocalScoutSortby                         : "<http://platform.bing.com/maps/localscout.sortby>",
        LocalScoutOpennow                        : "<http://platform.bing.com/maps/localscout.opennow>",
        LocalScoutDeals                          : "<http://platform.bing.com/maps/localscout.deals>",
        LocalScoutCategory                       : "<http://platform.bing.com/maps/localscout.category>",
        PoiCategoryEatDrink                      : "<http://platform.bing.com/maps/PoiCategory/EatDrink>",
        PoiCategorySeeDo                         : "<http://platform.bing.com/maps/PoiCategory/SeeDo>",
        PoiCategorySeeDoEvent                    : "<http://platform.bing.com/maps/PoiCategory/SeeDoEvent>",
        PoiCategoryShop                          : "<http://platform.bing.com/maps/PoiCategory/Shop>",
        BingMasterId                             : "<http://platform.bing.com/maps/Bing/masterId>",
        BingECoreId                              : "<http://platform.bing.com/maps/Bing/eCoreId>",
        BingSuggestId                            : "<http://platform.bing.com/maps/Bing/suggestId>",
        MemberOf                                 : "<http://schema.org/memberOf>",
        Members                                  : "<http://schema.org/members>",
        FavoritesCollection                      : "<http://platform.bing.com/maps/favorites.collection>",
        FavoritesSearch                          : "<http://platform.bing.com/maps/favorites.search>",
        FavoritesRegion                          : "<http://platform.bing.com/maps/favorites.region>",
        FavoritesMetric                          : "<http://platform.bing.com/maps/favorites.metric>",
        FavoritesShowBirdsEyeView                : "<http://platform.bing.com/maps/favorites.showBirdsEyeView>",
        DisplayOrder                             : "<http://schema.org/displayOrder>",
        MapZoomLevel                             : "<http://schema.org/mapZoomLevel>",
        Location                                 : "<http://schema.org/location>",
        BingEntityId                             : "<http://platform.bing.com/maps/preferences/favorites.entityId>",
        BingEntityUrl                            : "<http://platform.bing.com/maps/preferences/favorites.entityUrl>",
        Geo                                      : "<http://schema.org/geo>",
        RouteDetails                             : "<http://schema.org/routeDetails>",
        Address                                  : "<http://schema.org/address>",
        AddressSingleLine                        : "<http://schema.org/addressSingleLine>",
        Phone                                    : "<http://schema.org/phone>",
        ShowOnMap                                : "<http://schema.org/ShowOnMap>",
        TravelTime                               : "<http://platform.bing.com/maps/TravelTime>",
        OriginalName                             : "<http://platform.bing.com/maps/preferences/favorites.originalName>",
        FavoritesEntityType                      : "<http://platform.bing.com/maps/preferences/favorites.entityType>",
        FavoritesIsBusiness                      : "<http://platform.bing.com/maps/preferences/favorites.isBusiness>",
        BusinessType                             : "<http://platform.bing.com/maps/preferences/favorites.businessType>",
        FavoritesVersion                         : "<http://platform.bing.com/maps/preferences/favorites.version>",
        FavoritesCategory                        : "<http://platform.bing.com/maps/favorites.category>",
        FavoritesCategoryHome                    : "<http://platform.bing.com/maps/favorites.category/Home>",
        FavoritesCategoryWork                    : "<http://platform.bing.com/maps/favorites.category/Work>",
        FavoritesCategorySchool                  : "<http://platform.bing.com/maps/favorites.category/School>",
        FavoritesCategoryGym                     : "<http://platform.bing.com/maps/favorites.category/Gym>",
        FavoritesCategoryGrocery                 : "<http://platform.bing.com/maps/favorites.category/Grocery>",
        WifiIdentity                             : "<http://schema.org/wifiIdentity>",
        BluetoothIdentity                        : "<http://schema.org/bluetoothIdentity>",
        DisplayCoordinates                       : "<http://schema.org/displayCoordinates>",
        RouteFrom                                : "<http://platform.bing.com/maps/routeFrom>",
        RouteTo                                  : "<http://platform.bing.com/maps/routeTo>",
        WayPoint                                 : "<http://platform.bing.com/maps/wayPoint>",
        RouteMode                                : "<http://platform.bing.com/maps/routeMode>",
        RouteModeDriving                         : "<http://platform.bing.com/maps/routeMode/Driving>",
        RouteModeWalking                         : "<http://platform.bing.com/maps/routeMode/Walking>",
        RouteModeTransit                         : "<http://platform.bing.com/maps/routeMode/Transit>",
        RouteOption                              : "<http://platform.bing.com/maps/routeOption>",
        RouteOptionHighways                      : "<http://platform.bing.com/maps/routeOption/Highways>",
        RouteOptionTolls                         : "<http://platform.bing.com/maps/routeOption/Tolls>",
        RouteOptionMinimizeHighways              : "<http://platform.bing.com/maps/routeOption/MinimizeHighways>",
        RouteOptionMinimizeTolls                 : "<http://platform.bing.com/maps/routeOption/MinimizeTolls>",
        RouteOptionExpressTrain                  : "<http://platform.bing.com/maps/routeOption/ExpressTrain>",
        RouteOptionAirline                       : "<http://platform.bing.com/maps/routeOption/Airline>",
        RouteOptionBulletTrain                   : "<http://platform.bing.com/maps/routeOption/BulletTrain>",
        RouteTrafficAlerts                       : "<http://platform.bing.com/maps/routeTrafficAlerts>",
        TrafficRouteId                           : "<http://platform.bing.com/maps/trafficRouteId>",
        TrafficDataAvailable                     : "<http://platform.bing.com/maps/trafficDataAvailable>",
        TrafficRouteTravelTime                   : "<http://platform.bing.com/maps/trafficRouteTravelTime>",
        TrafficRouteSelectedHash                 : "<http://platform.bing.com/maps/trafficRouteSelectedHash>",
        TrafficAgentStatus                       : "<http://platform.bing.com/maps/trafficAgentStatus>",
        TrafficAgentStatusNoData                 : "<http://platform.bing.com/maps/trafficAgentStatus/NoData>",
        TrafficAgentStatusOff                    : "<http://platform.bing.com/maps/trafficAgentStatus/Off>",
        TrafficAgentStatusUpdating               : "<http://platform.bing.com/maps/trafficAgentStatus/Updating>",
        TrafficAgentStatusError                  : "<http://platform.bing.com/maps/trafficAgentStatus/Error>",
        TrafficAgentStatusOn                     : "<http://platform.bing.com/maps/trafficAgentStatus/On>",
        TrafficThreshold                         : "<http://platform.bing.com/maps/trafficThreshold>",
        TrafficSchedule                          : "<http://platform.bing.com/maps/TrafficSchedule>",
        TrafficAgentId                           : "<http://platform.bing.com/maps/trafficAgentId>",
        TrafficDeviceId                          : "<http://platform.bing.com/maps/trafficDeviceId>",
        TrafficTimeZone                          : "<http://platform.bing.com/maps/trafficTimeZone>",
        StreetAddress                            : "<http://schema.org/streetAddress>",
        PostalCode                               : "<http://schema.org/postalCode>",
        AddressRegion                            : "<http://schema.org/addressRegion>",
        AddressLocality                          : "<http://schema.org/addressLocality>",
        AddressCountry                           : "<http://schema.org/addresssCountry>",
        BuildingHeight                           : "<http://schema.org/buildingHeight>",
        Floors                                   : "<http://schema.org/floors>",
        Properties                               : "<http://platform.bing.com/maps/type.properties>",
        PropertyMaximumCardinality               : "<http://platform.bing.com/maps/type.MaxCardinality>",
        PropertyMinimumCardinality               : "<http://platform.bing.com/maps/type.MinCardinality>",
        PropertyExpectedType                     : "<http://platform.bing.com/maps/type.expected>",
        Login                                    : "<http://platform.bing.com/login>",
        MyDevice                                 : "<http://platform.bing.com/devices/me>",
        MyBingStore                              : "<http://platform.bing.com/devices/me/bingStore>",
        VersionHistoryLog                        : "<http://platform.bing.com/devices/me/bingStore/VersionHistoryLog>",
        LastPurgedTimestamp                      : "<http://platform.bing.com/devices/me/bingStore/LastPurgedTimestamp>",
        MinimumVersion                           : "<http://platform.bing.com/devices/me/bingStore/MinimumVersion>",
        Me                                       : "<http://platform.bing.com/persons/me>",
        MyPicturesLibrary                        : "<http://platform.bing.com/persons/me/pictures>",
        LocalScoutEntity                         : "<http://platform.bing.com/persons/me/preferences/localscout>",
        FavoritesEntity                          : "<http://platform.bing.com/persons/me/preferences/favorites>",
        CloudSyncEnrolled                        : "<http://platform.bing.com/persons/me/preferences/CloudSyncEnrolled>",
        SkyDriveStore                            : "<https://apis.live.net/v5.0/me/SkyDrive>",
        LocalStore                               : "<ms-knownFolders:///Pictures>",
        Live                                     : "<https://apis.live.net/v5.0>",
        ErrorMessage                             : "<http://platform.bing.com/maps/ErrorMessage>",
        StatusCode                               : "<http://platform.bing.com/maps/StatusCode>",
        Internal                                 : "<http://platform.bing.com/Internal>",
        InternalClientImportVersion              : "<http://platform.bing.com/Internal/ClientImportVersion>",
        InternalDoFullSave                       : "<http://platform.bing.com/Internal/DoFullSave>",
        InternalIsPurgeDisabled                  : "<http://platform.bing.com/Internal/IsPurgeDisabled>",
        InternalServerVersionFromStore           : "<http://platform.bing.com/Internal/ServerVersionFromStore>",
        InternalPersistentState                  : "<http://platform.bing.com/Internal/PersistentState>",
        InternalDiagnosticsLevel                 : "<http://platform.bing.com/Internal/Diagnostics/Level>",
        InternalDiagnosticsMessage               : "<http://platform.bing.com/Internal/Diagnostics/Message>",
        InternalLoadFrom                         : "<http://platform.bing.com/Internal/Store/LoadFrom>",
        InternalRetrieveLocationListResponse     : "<http://platform.bing.com/Internal/RetrieveLocationListResponse>",
        InternalSuggestionResponse               : "<http://platform.bing.com/Internal/SuggestionResponse>",
        FavoritesDefaultCollection               : "<http://platform.bing.com/persons/me/geoAnnotationCollection.default>",
        FavoritesRecentCollection                : "<http://platform.bing.com/persons/me/geoAnnotationCollection.recent>",
        FavoritesDefaultCollectionList           : "<http://platform.bing.com/persons/me/list.geoAnnotationCollection.default>",
        FavoritesRecentCollectionList            : "<http://platform.bing.com/persons/me/list.geoAnnotationCollection.recent>",
        MyListNode                               : "<http://platform.bing.com/persons/me/list.>",
        MyStatement                              : "<http://platform.bing.com/persons/me/statement>",
        WRPMigratedFavorites                     : "<http://platform.bing.com/maps/WRP.MigratedFavorites>",
        WRPMigrateStartTime                      : "<http://platform.bing.com/maps/WRP.MigrateStartTime>",
        MyInternal                               : "<http://platform.bing.com/persons/me/Internal>",
        MasterCollection                         : "<http://platform.bing.com/persons/me/Internal/MasterCollection>",
        PreferredCollection                      : "<http://platform.bing.com/persons/me/Internal/PreferredCollection>",
        ProtectedState                           : "<http://platform.bing.com/protected/State>",
        ProtectedExceptionMessage                : "<http://platform.bing.com/protected/State/ExceptionMessage>",
        ProtectedExceptionStackTrace             : "<http://platform.bing.com/protected/State/ExceptionStackTrace>"
    }
    
    //      _____                       _
    //     | ____|_  ___ __   ___  _ __| |_ ___
    //     |  _| \ \/ / '_ \ / _ \| '__| __/ __|
    //     | |___ >  <| |_) | (_) | |  | |_\__ \
    //     |_____/_/\_\ .__/ \___/|_|   \__|___/
    //                |_|
    
    module.exports = enums;

// DO NOT PLACE ANYTHING BELOW THE FOLLOWING LINE -- IT
// BRACKETS THE ENTIRE FILE.
})();
