// ----------------------------------------------------------------------------------------------------------------------
//   Copyright 2014 (c) Microsoft Corporation.
//   Summary: Defines the console Interpreter.
// ----------------------------------------------------------------------------------------------------------------------
(function () {
    "use strict";  // whole file

    //  ___                            _
    // |_ _|_ __ ___  _ __   ___  _ __| |_ ___
    //  | || '_ ` _ \| '_ \ / _ \| '__| __/ __|
    //  | || | | | | | |_) | (_) | |  | |_\__ \
    // |___|_| |_| |_| .__/ \___/|_|   \__|___/
    //               |_|

    var graph = require("./Graph");

    //  ___       _                           _
    // |_ _|_ __ | |_ ___ _ __ _ __  _ __ ___| |_ ___ _ __
    //  | || '_ \| __/ _ \ '__| '_ \| '__/ _ \ __/ _ \ '__|
    //  | || | | | ||  __/ |  | |_) | | |  __/ ||  __/ |
    // |___|_| |_|\__\___|_|  | .__/|_|  \___|\__\___|_|
    //                        |_|

    var interpreter = {
        line: 0,

        engine: graph.CreateGraph(),

        log: function () {
            this.engine.Log.apply(undefined, arguments);
        },

        start: function () {
            this.log({ "#green": "(* Interpreter bootup *)" });
            process.stdin.resume();
            process.stdin.setEncoding("utf8");
            this.prompt();

            process.stdin.on("data", this._thunk(this.onNext));
            process.stdin.on("end", this._thunk(this.onCompleted));
        },

        prompt: function () {
            process.stdout.write(this.engine.LogFormat({ "#yellow": "In[{0}]:= "._format(++this.line) }));
        },

        onNext: function (chunk) {
            if (chunk.match(/\cZ/)) {
                this.onCompleted();
            }
            if (chunk.replace(/\s*/, "") !== "") {
                this.engine.Context.SetCallback(this);
                this.engine.Evaluate(chunk);
            }
        },

        onCompleted: function (chunk) {
            process.exit(0);
        },

        // Callbacks from engine
        OnCompleted: function (expr) {
            expr.Parts.slice(1)._forEach(this._thunk(function (k, v) {
                if (v.Head.Symbol === "MessagePacket") {
                    this.log({ "#red": v.Parts[1].toStandardForm() });
                    return;
                } else if (v.Head.Symbol === "TextPacket") {
                    this.log(v.Parts[1].toStandardForm());
                    return;
                } else if (v.Head.Symbol === "TracePacket") {
                    this.log({ "#yellow": v.Parts[1].toStandardForm() });
                    return;
                }
                var s = v.ToString();
                if (s !== "Null") {
                    this.log({ "#yellow": "Out[{0}]="._format(this.line) }, { "#bold": s });
                }
            }));
            this.prompt();
        }
    };

    //  _____                       _
    // | ____|_  ___ __   ___  _ __| |_ ___
    // |  _| \ \/ / '_ \ / _ \| '__| __/ __|
    // | |___ >  <| |_) | (_) | |  | |_\__ \
    // |_____/_/\_\ .__/ \___/|_|   \__|___/
    //            |_|

    module.exports.interpreter = interpreter;

    // DO NOT PLACE ANYTHING BELOW THE FOLLOWING LINE -- IT
    // BRACKETS THE ENTIRE FILE.
})();
