// ----------------------------------------------------------------------------------------------------------------------
//   Copyright 2014 (c) Microsoft Corporation.
//   Summary: Defines the Graph Logger.
// ----------------------------------------------------------------------------------------------------------------------
(function () {
    "use strict";

    //  _                      _
    // | |    ___   __ _  __ _(_)_ __   __ _
    // | |   / _ \ / _` |/ _` | | '_ \ / _` |
    // | |__| (_) | (_| | (_| | | | | | (_| |
    // |_____\___/ \__, |\__, |_|_| |_|\__, |
    //             |___/ |___/         |___/

    var logger = {
        logging: false,

        start: function () {
            this.colorTable = typeof (process) !== "undefined" ? this.colorTableConsole : this.colorTableBrowser;

            if (this.logging || typeof (console) === "undefined") {
                return;
            }
            this.logging = true;
        },

        stop: function () {
            this.logging = false;
        },

        log: function () {
            if (this.logging) {
                console.log(this.toString.apply(this, arguments));
            }
        },

        colorTableConsole : {
            "#bold":      "\u001b[1m" + "{0}\u001b[0m",
            "#italic":    "\u001b[3m" + "{0}\u001b[0m",
            "#underline": "\u001b[4m" + "{0}\u001b[0m",
            "#black":     "\u001b[30m"+ "{0}\u001b[0m",
            "#red":       "\u001b[31m"+ "{0}\u001b[0m",
            "#green":     "\u001b[32m"+ "{0}\u001b[0m",
            "#yellow":    "\u001b[33m"+ "{0}\u001b[0m",
            "#blue":      "\u001b[34m"+ "{0}\u001b[0m",
            "#magenta":   "\u001b[35m"+ "{0}\u001b[0m",
            "#cyan":      "\u001b[36m"+ "{0}\u001b[0m",
            "#white":     "\u001b[37m"+ "{0}\u001b[0m"
        },

        colorTableBrowser: {
            "#bold":        "<b>{0}</b>",
            "#italic":      "<i>{0}</i>",
            "#underline":   "<u>{0}</u>",
            "#black":       "<font color=\"black\">{0}</font>",
            "#red":         "<font color=\"red\">{0}</font>",
            "#green":       "<font color=\"green\">{0}</font>",
            "#yellow":      "<font color=\"yellow\">{0}</font>",
            "#blue":        "<font color=\"blue\">{0}</font>",
            "#magenta":     "<font color=\"magenta\">{0}</font>",
            "#cyan":        "<font color=\"cyan\">{0}</font>",
            "#white":       "<font color=\"white\">{0}</font>"
        },

        toString: function () {
            var output = "";
            var last = arguments.length-1;

            arguments._forEach(
                function (n, v) {
                    var item;
                    var sub = [];
                    if (typeof (v) === "undefined") {
                        item = "undefined";
                    }  else if (v instanceof (Array)) {
                        v._forEach(function (k, v) { sub.push(logger.toString(v)); });
                        item = "[ " + sub.join(", ") + " ]";
                    } else if (v.isJacquard) {
                        item = v.toInputForm();
                    } else if (typeof(v) === "function") {
                        item = "function(){...}";
                    } else if (v instanceof (Object)) {
                        var colorCount = 0;
                        v._forEach(function (k, v) {
                            var subst = logger.colorTable[k];
                            if (subst) {
                                colorCount++;
                                subst = subst._format(logger.toString(v));
                                sub.push(subst);
                            } else {
                                sub.push(k + ": " + logger.toString(v));
                            }
                        });
                        item = colorCount === 1 && sub.length === 1 ? sub[0] : "{ " + sub.join(", ") + " }";
                    } else {
                        item = v;
                    }

                    output += item + (n < last ? " " : "");
                }
            );

            return output;
        }
    };

    logger.start();

    //  _____                       _
    // | ____|_  ___ __   ___  _ __| |_ ___
    // |  _| \ \/ / '_ \ / _ \| '__| __/ __|
    // | |___ >  <| |_) | (_) | |  | |_\__ \
    // |_____/_/\_\ .__/ \___/|_|   \__|___/
    //            |_|
    module.exports.log = function () { logger.log.apply(logger, arguments); };
    module.exports.toString = function () { return logger.toString.apply(logger, arguments); };

    // DO NOT PLACE ANYTHING BELOW THE FOLLOWING LINE -- IT
    // BRACKETS THE ENTIRE FILE.
})();
