// ----------------------------------------------------------------------------------------------------------------------
//   Copyright 2014 (c) Microsoft Corporation.
//   Summary: Defines Tests for the Graph.  INTERNAL USE ONLY, NOT TO BE USED AS AN API REFERENCE. USE SAMPLE.JS INSTEAD.
// ----------------------------------------------------------------------------------------------------------------------
(function () {
    "use strict"; // whole file
    
    var sample = require("./Sample");
    var LogTerm = sample.LogTerm;

    //      _____         _
    //     |_   _|__  ___| |_ ___
    //       | |/ _ \/ __| __/ __|
    //       | |  __/\__ \ |_\__ \
    //       |_|\___||___/\__|___/
    //
    function Tests(graph) {
        // setInterval(function () { graph.Sync(function () { graph.Log({ "#green": "Syncing again..." }); }); }, 5000);
        
        // TODO: For future rev, add testCase and let it keep count of #tests, #failed.
        graph._testsPassed = true;
        graph._extends("_setTestStatus", function (status, message) {
            this._testsPassed = this._testsPassed && status;
            if (message && typeof (message) === "string") {
                this.Log(status === true ? { "#green": message + ": " + status } : { "#red": message + ": " +status });
            }
        });

        graph.Log(graph.CreateText("hello\n'world").Id);
        graph.Log(graph.CreateText("hello\n'world").Value);
        graph.Log(graph.CreateText("hello" + String.fromCharCode(0x202A)).Id);

        graph.Log(graph.CreateGeometryPoint(35.5, 45.5).Id);
        graph.Log(graph.CreateGeometryPoint(35.5, 45.5, 1.3).Id);
        graph.Log(graph.CreateGeometryPoint(35.5, 45.5, undefined, 7).Id);
        graph.Log(graph.CreateGeometryPoint(35.5, 45.5).Type.Id);

        graph.Log("Test GUID: " + String._createGuid());

        // If graph loaded, we should this stitched.
        graph.Log(graph.Me.Preferences.Favorites.Type.Id);
        graph.Log(graph.Me.Type);
        
        graph.Log(graph.Entities.MyStatement.IsSubPathOf(graph.Me.Id));
        
        var namespaces = new require("./Tokenizer").MapNamespaces(graph, true);
        graph.Log(namespaces.Compress(graph.Me.Id));
        graph.Log(namespaces.Compress('"Point(35.5 44.5)"^^<http://www.opengis.net/ont/geosparql#wktLiteral>'));
        graph.Log(namespaces.Expand("p:me"));
        graph.Log(namespaces.Expand('"Point(35.5 44.5)"^^geo:wktLiteral'));
        
        graph.Me.Out(graph.Entities.MyStatement).Append(graph.Create("\"Hello\""));
        graph.Me.Out(graph.Entities.Name).Append(graph.Create("\"Names\""));
        graph.Log("Keys", graph.Me.OutKeys.map(function (x) { return x.Id }));
        graph.Me.OutKeys._forEach(function (k, v) {
            graph.Log(v.Id);
            var s = graph.Me.Out(v);
            graph.Log(s.Parent.Id);
        });
        
        var i = graph.Create("5.5");
        graph.Log(i.Type);
        graph._setTestStatus(i._isOrphaned, "i._isOrphaned");
        
        var d = graph.Create("\"2013-11-21T00:03:58.022Z\"^^<http://schema.org/Date>");
        graph.Log(d.Type);
        
        var d = graph.Create("true");
        graph.Log(d.Type);
        
        var d = graph.Create('"""Text\\r\\nValue"""');
        graph.Log(d.Type);
        graph.Log(d.Value);
        
        graph.Me.Out(graph.Entities.Type).Append(graph.Entities.Person);
        
        graph.Log(graph.Me.Type.Id);
        
        var d = graph.Create('"Point(35.5 44.5)"^^<http://www.opengis.net/ont/geosparql#wktLiteral>');
        graph.Log("Stop");

        graph.Sync(function () {
            // Note: this is an async call.
            graph.Log("Sync completed.");
            
            var home = graph.Me.Preferences.Favorites.GetPreferredPlace(graph.Entities.FavoritesCategoryHome);
            if (home) {
                graph.Log("Home: " + home.OriginalName);
                var lat = home.DisplayCoordinates === undefined ? home.Geometry.Latitude.Value : home.DisplayCoordinates.Latitude.Value;
                graph.Log(lat);
            } else {
                graph.Log({ "#yellow": "No Home found, creating one." });
                home = graph.Me.Preferences.Favorites.Collections.Default.CreatePlace("newHome");
                home.BingEntityId = graph.CreateText("New Home Entity Id");
                home.OriginalName = graph.CreateText("New Home OriginalName");
                home.Name = graph.CreateText("New Home Name");
                home.Geometry = graph.CreateGeometryPoint(1.2, 34.5, 6.0, 7);
                graph.Me.Preferences.Favorites.SetPreferredPlace(graph.Entities.FavoritesCategoryHome, home);
            }

            var tempHome = graph.Me.Preferences.Favorites.Collections.Default.CreatePlace("tempHome");
            tempHome.BingEntityId = graph.CreateText("Temp Home Entity Id");
            tempHome.OriginalName = graph.CreateText("Temp Home OriginalName");
            tempHome.Name = graph.CreateText("Temp Home Name");
            tempHome.Geometry = graph.CreateGeometryPoint(1.2, 34.5, 6.0, 7);
            graph.Me.Preferences.Favorites.SetPreferredPlace(graph.Entities.FavoritesCategoryHome, tempHome);

            var newHome = graph.Me.Preferences.Favorites.GetPreferredPlace(graph.Entities.FavoritesCategoryHome);
            graph.Log("NewHome: " + newHome.OriginalName);
            graph.Me.Preferences.Favorites.SetPreferredPlace(graph.Entities.FavoritesCategoryHome, home);
            graph._setTestStatus(tempHome === newHome, "tempHome === newHome");

            var revertedHome = graph.Me.Preferences.Favorites.GetPreferredPlace(graph.Entities.FavoritesCategoryHome);
            graph.Log("RevertedHome: " + revertedHome.OriginalName);
            graph._setTestStatus(revertedHome === home, "revertedHome === home");   

            var cols = graph.Me.Preferences.Favorites.Collections;
            graph.Log(cols.Count);
            graph.Log(cols.Default.Id);
            
            var members = cols.Default.Members;
            LogTerm(cols.Default)
            
            //graph.Log("Removing at 6");
            //cols.Default.Members.RemoveAt(6);
            //LogTerm(cols.Default)
            
            // Place Creation and update tests
            var d = graph.Me.Preferences.Favorites.Collections.Default.CreatePlace("fakeDefaultPlaceId");
            graph.Log("Created a Default Place: " + d.Id);
            d.BingEntityId = graph.CreateText("Default Place Entity Id 1");
            d.Address = graph.CreateText("1 Default Address Street");
            d.Phone = graph.CreateText("111-DEF-AULT");
            d.OriginalName = graph.CreateText("fake Default Place 1");
            d.BusinessType = graph.CreateText("fake Default Business Place 1");
            d.DisplayCoordinates = graph.CreateGeometryPoint(1.1, 11.1, 1.0, 1);
            LogTerm(d);
            graph._setTestStatus(d._isOrphaned === false, "    !_isOrphaned");

            graph.Log("Making changes to the Default Place: " + d.Id);
            d.BingEntityId = graph.CreateText("Default Place Entity Id 2");
            d.Address = graph.CreateText("2 Default Address Street");
            d.Phone = graph.CreateText("222-DEF-AULT");
            d.OriginalName = graph.CreateText("fake Default Place 2");
            d.BusinessType = graph.CreateText("fake Default Business Place 2");
            d.Geometry = graph.CreateGeometryPoint(2.2, 22.2, 2.0, 2);
            LogTerm(d);

            var d = graph.Me.Preferences.Favorites.Collections.Recent.CreatePlace("fakeRecentPlaceId");
            graph.Log("Created a Recent Place: " + d.Id);
            d.BingEntityId = graph.CreateText("Recent Place Entity Id 1");
            d.Address = graph.CreateText("1 Recent Address Street");
            d.Phone = graph.CreateText("555-5RE-CENT");
            d.OriginalName = graph.CreateText("fake Recent Place");
            d.BusinessType = graph.CreateText("fake Recent Business Place");
            d.Geometry = graph.CreateGeometryPoint(2.2, 22.2, 2.0, 2);
            LogTerm(d);
            
            d.Delete();

            var d = graph.Me.Preferences.Favorites.Collections.Default.CreateRoute("fakeDefaultRouteId");
            graph.Log("Created a Default Route: " + d.Id);
            d.BingEntityId = graph.CreateText("Default Route Entity Id 1");
            d.Name = graph.CreateText("Default Route Name 1");
            var wp1 = graph.Me.Preferences.Favorites.Collections.Default.CreatePlace("Waypoint1");
            wp1.BingEntityId = graph.CreateText("Waypoint Place Entity Id 1");
            wp1.Address = graph.CreateText("1 Waypoint Address Street");
            wp1.Phone = graph.CreateText("55W-AYP-OINT");
            wp1.OriginalName = graph.CreateText("fake Waypoint Place 1");
            wp1.BusinessType = graph.CreateText("fake Waypoint Business Place 1");
            wp1.Geometry = graph.CreateGeometryPoint(1.1, 11.1, 1.0, 1);
            var wp2 = graph.Me.Preferences.Favorites.Collections.Default.CreatePlace("Waypoint2");
            wp2.BingEntityId = graph.CreateText("Waypoint Place Entity Id 2");
            wp2.Address = graph.CreateText("2 Waypoint Address Street");
            wp2.Phone = graph.CreateText("55W-AYP-OINT");
            wp2.OriginalName = graph.CreateText("fake Waypoint Place 2");
            wp2.BusinessType = graph.CreateText("fake Waypoint Business Place 2");
            wp2.DisplayCoordinates = graph.CreateGeometryPoint(2.2, 22.2, 2.0, 2);
            d.Waypoints.Append(wp1);
            d.Waypoints.Append(wp2);
            LogTerm(d);
            
            var wp3 = graph.Me.Preferences.Favorites.Collections.Default.CreatePlace("Waypoint3");
            wp3.BingEntityId = graph.CreateText("Waypoint Place Entity Id 3");
            wp3.Address = graph.CreateText("3 Waypoint Address Street");
            wp3.Phone = graph.CreateText("55W-AYP-OINT");
            wp3.OriginalName = graph.CreateText("fake Waypoint Place 3");
            wp3.BusinessType = graph.CreateText("fake Waypoint Business Place 3");
            wp3.Geometry = graph.CreateGeometryPoint(3.3, 33.3, 3.0, 3);
            d.Waypoints.Append(wp3);
            LogTerm(d);
            
            d.Waypoints.Remove(wp2);
            LogTerm(d);

            graph.Me.Preferences.Favorites.Collections.Default.Members.Remove(wp1);
            d.Delete();

            graph.Log(graph._testsPassed === true ? { "#green": "Tests Passed." } : { "#red": "Tests Failed." });

            sample.Sample(graph);
        });

    };

    //      _____                       _
    //     | ____|_  ___ __   ___  _ __| |_ ___
    //     |  _| \ \/ / '_ \ / _ \| '__| __/ __|
    //     | |___ >  <| |_) | (_) | |  | |_\__ \
    //     |_____/_/\_\ .__/ \___/|_|   \__|___/
    //                |_|
    
    module.exports = Tests;
    
    // DO NOT PLACE ANYTHING BELOW THE FOLLOWING LINE -- IT
    // BRACKETS THE ENTIRE FILE.
})();
