/// <disable>JS2043.RemoveDebugCode</disable>
/// <disable>JS2087.DoNotDeleteObjectProperties</disable>

var events = require('events');
var url = require('url');

var tag = 'answers/answers.js';

var answersRefresh = new events.EventEmitter();
answersRefresh.lastResults = null;
answersRefresh.lastError = null;
answersRefresh.inProgress = false;

answersRefresh.start = function () {
  answersRefresh.inProgress = true;
  answersRefresh.emit('start');
}

answersRefresh.done = function (err, results) {
  answersRefresh.lastResults = results;
  answersRefresh.lastError = err;
  answersRefresh.inProgress = false;
  answersRefresh.emit('done', err, results);
}

exports.start = function start(router, proactive) {

  console.log('%s: adding GET handler for [/proactive*]', tag);

  // Returns the Following HTTP Status Codes:
  //  200 OK - base page is returned either from cache or the server
  //  504 GATEWAY_TIMEOUT - timed out while waiting for a response from the server.
  //  503 GATEWAY_UNAVAILABLE - failed to make a request to the server due to the lack of internet connection.
  //  500 INTERNAL_ERROR - all other internal errors.
  router.get('/proactive*', function getProactive(req, res) {
    console.log('%s: Handling GET request for [%d]', tag, req.__reqId);

    // TODO: Figure out what to do if a refresh is already in progress
    answersRefresh.start();

    var params = getParamsFromRequest(req);
    proactive.getAnswers(params, res, function (err, results) {
      if (err) {
        console.error('Error in GetAnswers: ', err);

        // TODO:
        //   Standarize error messages coming from proactive DSS in terms of template, language and localization.
        var retStatusCode = 500;
        // return the description of the error to the caller.
        var message = err.description;

        if (err.name == 'NoConnectionError') {
          // special-case: NoConnectionError
          // Indicate to the caller that the request failed due to the absence of internet connectivity.
          retStatusCode = 503;
        }
        else if (err.name == 'RequestTimeoutError') {
          // special-case: RequestTimeoutError
          // Indicate to the caller that the local proactive service is unable to reach bing.com.
          retStatusCode = 504;
        }

        res.writeHead(retStatusCode, { 'Content-Type': 'text/html' });
        // end the response
        res.end(message);
        return answersRefresh.done(err);
      }
// todo: res.end(); ????????????????
      return answersRefresh.done(null, results);
    });
  });

  console.log('%s: adding GET handler for [/answers*]', tag);
  router.get('/answers*', function getAnswers(req, res) {
    console.log('%s: Handling GET request for [%d]', tag, req.__reqId);
    if (req.headers && req.headers['accept'] == 'application/json') {

      // Returns the server-generated and client-generated answers in JSON format
      function returnAnswersAsJSON(err, results) {
        // Check whether we encountered an error during the refresh
        if (err) {
          console.error(err);
          res.writeHead(500, { 'Content-Type': 'text/html' });
          return res.end(err.message);
        }

        // Make sure we have a set of answers (even if it's empty) and return it
        res.writeHead(200, { 'Content-Type': 'application/json' });
        return res.end(JSON.stringify(results));
      }
      // Only return once the answers are ready
      if (answersRefresh.inProgress) {
        answersRefresh.once('done', returnAnswersAsJSON);
      }
      else {
        returnAnswersAsJSON(answersRefresh.lastError, answersRefresh.lastResults);
      }
    }
    else {
      res.writeHead(500, { 'Content-Type': 'text/html' });
      return res.end('Only supported content type is application/json');
    }
  });
};

// Extracts request parameters from request headers
function getParamsFromRequest(req) {
  var params = {};
  if (req) {
    if (req.headers) {
      if ('x-bm-devicedimensions' in req.headers) {
        params.viewDimension = req.headers['x-bm-devicedimensions'];
      }
      else {
        console.warn('%s: Request is missing x-bm-devicedimensions.', tag);
      }

      if ('x-bm-theme' in req.headers) {
        params.viewTheme = req.headers['x-bm-theme'];
      }
      else {
        console.warn('%s: Request is missing x-bm-theme.', tag);
      }

      if ('user-agent' in req.headers) {
        params.userAgent = req.headers['user-agent'];
      }
      else {
        console.warn('%s: Request is missing user-agent.', tag);
      }
    }

    var requestUrl = url.parse(req.url, true);
    if (requestUrl && requestUrl.query) {
	  //extracting "form" parameter
      if (requestUrl.query['form']) {
        params.appFormCode = requestUrl.query['form'];
      }
      else {
        console.warn('%s: Request query is missing form code. ', tag);
      }
	  
	  //extracting "topid" parameter
	  if (requestUrl.query['topid']) {
        params.topID = requestUrl.query['topid'];
      }
      else {
        console.log('%s: Request query is missing topid code. ', tag);
      }
    }
    else {
      console.log('%s: Url is missing query object. ', tag);
    }
  }

  return params;
}