var fs = require('fs');
var path = require('path');

var tag = 'test/answersMocks'

// MockRouter:
//  Contains a stub for GET route registration
var MockRouter = function MockRouter() {
  if (!(this instanceof (MockRouter))) {
    return new MockRouter();
  }
  // contains [route_name:function] pairs for
  // registered routes.
  this.routeHandlers = {};
};

// stub for router.get(..)
MockRouter.prototype.get = function (name, cb) {
  console.log('%s: Get [%s, %s]', tag, name, typeof (cb));
  
  // special case known proactive routes to avoid having
  // special characters in key names.
  if (name == '/proactive*') {
    this.routeHandlers.proactiveGetHandler = cb;
  }
  else if (name == '/answers*') {
    this.routeHandlers.answersGetHandler = cb;
  }
  else {
    // TODO: throw?
    console.error('%s: Route name [%s] is unknown.', tag, name);
  }
};

// Export MockRouter
//
exports.MockRouter = MockRouter;


// MockProactive
//  Constains a stub method for proactive.getAnswers(..)
var MockProactive = function MockProactive() {
  if (!(this instanceof (MockProactive))) {
    return new MockProactive();
  }
  
  // list of common getAnswers behaviors.
  this.getProactiveBehaviors = {
    noError : {
      errorName: null,
      results: getMockAnswersSync(),
      basePage: 'Dummy basepage content..'
    },
    noConnection: {
      errorName: 'NoConnectionError',
      results: null,
      basePage: null
    },
    requestTimeout: {
      errorName: 'RequestTimeoutError',
      results: null,
      basePage: null,
    },
    genericError: {
      errorName: 'GenericError',
      results: null,
      basePage: null,
    },
    invalidState: {
      errorName: null,
      results: null,
      basePage: null,
    }
  };
};

// Sets an expected behavior for proactive.getAnswers(..)
MockProactive.prototype.setGetProactiveBehavior = function (behaviorName) {
  if (behaviorName in this.getProactiveBehaviors) {
    // set getAnswers behavior to requested one
    MockProactive.prototype.getAnswers = function (params, res, cb) {
      var targetBehavior = this.getProactiveBehaviors[behaviorName];
      var retResults = targetBehavior.results;
      var retError = null;
      
      if (targetBehavior.errorName) {
        retError = new Error('Something Went Wrong');
        retError.name = targetBehavior.errorName;
      }
      
      // retResults will contain the answers part (2nd part) of the payload
      // we still need a way to commit the base page to the response stream
      if (targetBehavior.basePage) {
        res.end(targetBehavior.basePage);
      }
      
      process.nextTick(function () {
        return cb(retError, retResults);
      });
    }
  }
  else {
    console.error('%s: Behavior [%s] is unknown.', tag, behaviorName);
    // TODO: throw?
  }
};

// export mock proactive
exports.MockProactive = MockProactive;


// blocking call, reads from a file and returns json object
var getMockAnswersSync = function () {
  var jsonContent = fs.readFileSync(path.resolve(__dirname, 'answersPayloadMock.json'));
  return JSON.parse(jsonContent);
};

exports.getMockAnswersSync = getMockAnswersSync;