var util = require('util');
var EventEmitter = require('events').EventEmitter;
var stream = require('stream');

// MockServerResponse
//  Exposes the standaed writable Stream methods in addition to:
//  writeHead, getHeaders, getStatusCode, getStatusMessage, getContent, getContentType
var MockServerResponse = function MockServerResponse() {
  if (!(this instanceof (MockServerResponse))) {
    return new MockServerResponse();
  }
  
  stream.Writable.call(this);
  // init with some default values
  this._dataBuffer = [];
  this._statusCode = 200;
  this._statusMessage = '';
  this._headers = [];
  this._contentType = 'text/html';
};

// inherit some more stuff..
util.inherits(MockServerResponse, stream.Writable);

// Chunk is a buffer, using default stream.Writable state.
// Ignores encoding
MockServerResponse.prototype._write = function (chunk, encoding, callback) {
  this._dataBuffer.push(chunk);
  callback();
};

MockServerResponse.prototype.getStatusCode = function () {
  return this._statusCode;
};

MockServerResponse.prototype.getStatusMessage = function () {
  return this._statusMessage;
};

MockServerResponse.prototype.getContent = function () {
  return Buffer.concat(this._dataBuffer);
};

MockServerResponse.prototype.getHeaders = function () {
  return this._headers;
};

MockServerResponse.prototype.getContentType = function () {
  return this._contentType;
};

MockServerResponse.prototype.writeHead = function (statusCode, statusMessage, headers) {
  this._statusCode = statusCode;
  
  var headersObj = headers;
  // only accept string status message
  if (typeof statusMessage === 'string') {
    this._statusMessage = statusMessage;
    this._write(statusMessage);
  }
  else {
    this._statusMessage = 'Unknown';
    // support optioanl parameters
    headersObj = statusMessage;
  }
  
  if (this._headers) {
    if (headersObj) {
      var keys = Object.keys(headersObj);
      for (var i = 0; i < keys.length; i++) {
        var key = keys[i];
        var value = headersObj[key];
        
        // for each key.. keys[i]
        if (value && key.toUpperCase() == 'CONTENT-TYPE') {
          this._contentType = value;
        }
        
        if (value) {
          this._headers[key] = value;
        }

      }      ;
    }
  }
  else {
    this._headers = headersObj;
  }
};

// Export a method to create MockServerResponse
//
exports.createMockServerResponse = function createMockServerResponse() {
  return new MockServerResponse();
};

// MockRequest
//
var MockRequest = function MockRequest() {
  if (!(this instanceof (MockRequest))) {
    return new MockRequest();
  }
  
  this.__reqId = new Date().getTime();
  this.headers = [];
};

// Export a method to create MockRequest
//
exports.createMockRequest = function () {
  return new MockRequest();
};