/// <disable>JS2043.RemoveDebugCode</disable>
/// <disable>JS2064.SpecifyNewWhenCallingConstructor</disable>

var tag = 'providers.js';
var clientExperienceProviders = require('./providersRegistry');
var url = require('url');

try {
  // Include SDK providers if available
  var path = require('path');
  var fs = require('fs');
  var providersFromSdkPath = path.join(__dirname, '../../answers/providers_sdk.js');
  if (fs.existsSync(providersFromSdkPath)) {
    var providersFromSdk = require(providersFromSdkPath);
    providersFromSdk.extendProviders(clientExperienceProviders);
  }
  // Include test providers if available
  var testProvidersPath = path.join(__dirname, '../test/test_providers.js');
  if (fs.existsSync(testProvidersPath)) {
    var testProviders = require(testProvidersPath);
    testProviders.addTestProviders(clientExperienceProviders);
  }
} 
catch (err) {
  // Ignore exception the app is running without extra providers
}

// providers dictionary object
exports.clientExperienceProviders = clientExperienceProviders;

// starts providers http services 
exports.start = function start(router) {
  // Create a GET handler function for each of the providers
  for (var path in clientExperienceProviders) {
    
    var provider = clientExperienceProviders[path];
    var basepath = '/' + path;
    var handler = (function (provider) {
      return function (req, res) {
        console.log('%s: Handling GET request for [%d]', tag, req.__reqId);
        
        var parsedUrl = url.parse(req.url, true);
        var options = parsedUrl.query;
        
        return provider.produce(options, function (err, data, contentType) {
          
          if (err) {
            console.error(err);
            res.writeHead(500, { 'Content-Type': 'text/plain' });
            return res.end(err.message);
          }
          
          res.writeHead(200, { 'Content-Type': contentType || 'application/json' });
          return res.end((typeof data === 'object') ? JSON.stringify(data) : data);
        });
      };
    })(provider);
    
    console.log('%s: adding GET handler for [%d]', tag, basepath);
    router.add(basepath, handler);
    if (provider.serve) {
      console.log('%s: adding %s to router with provider.serve for answers', tag, basepath);
      router.add(basepath + '/*', provider.serve);
      provider.__basepath__ = basepath;
    }
  }
}

