var java = require('java');
var logger = require('logger')(__filename);

var ENDPOINT_SIGNALS = 'signals';
var REGISTRATION_ID = 'registrationId';
var SNR = 'snr';
var REMINDER_SERVICE = 'reminderService';
var ENDPOINT_ENTITY = 'entity';
var ENDPOINT_PROACTIVE = 'proactive';

function getHeaders(options, cb) {

  if(!cb) throw new Error('callback was not provided');

  if(!options || !options.endpoint) throw new Error('endpoint was not provided');
  var endpoint = options.endpoint;

  var callback = java.createJavaProxy('com.microsoft.bing.dss.servicelib.components.HeadersCallback', {
    onHeaders: function(err, headersList) {
      if (err) {
        logger.error('failed to get headers. ', err.toString());
        return cb(err);
      }

      if (!headersList) {
        var msg = 'headers list is null';
        logger.error(msg);
        return cb(msg);
      }

      var headers = {};
      for (var i = 0; i < headersList.length; i++) {
        var header = headersList[i];
        headers[header.getName()] = header.getValue();
      }

      logger.log('Headers List: ', JSON.stringify(headers));
      cb(null, headers);
    }
  });

  if (endpoint === ENDPOINT_SIGNALS) {
    java.device.headers.getSignalsHeaders(callback);
  }
  else if (endpoint === ENDPOINT_ENTITY) {
    java.device.headers.getEntityHeaders(callback);
  }
  else if (endpoint === REGISTRATION_ID) {
    java.device.headers.getRegistrationHeaders(callback);
  }
  else if (endpoint === SNR) {
    java.device.headers.getSnrHeaders(callback);
  }
  else if (endpoint === REMINDER_SERVICE) {
    java.device.headers.getReminderServiceHeaders(callback);
  }
}

function setBingCookies(cookies) {
  java.device.headers.setBingCookies(cookies);
}

module.exports = {
  getHeaders: getHeaders,
  setBingCookies: setBingCookies,
  ENDPOINT_SIGNALS: ENDPOINT_SIGNALS,
  REGISTRATION_ID: REGISTRATION_ID,
  SNR: SNR,
  REMINDER_SERVICE: REMINDER_SERVICE,
  ENDPOINT_ENTITY: ENDPOINT_ENTITY
}