var headersHelper = require('./headersHelper');

var util = require('util');
var isAndroid = (process.platform == 'android');
var logger = isAndroid ? require('logger')(__filename) : console;
var java = require('java');

var ANDROID_X_BM_DATE_FORMAT = 'M/d/yyyy';
var ANDROID_X_DEVICE_PRODUCT = 'Virtual Machine';
var ANDROID_X_DEVICE_MANUFACTURER = 'Microsoft Corporation';
var ANDROID_X_DEVICE_SKU = 4;
var ANDROID_X_DEVICE_OS_SKU = 4;
var ANDROID_X_DEVICE_ID = 'DCDF5E78629E3C0BEA1CD8A91DB7202955B0F4C73087B85275596D353D30B580';
var ANDROID_X_DEVICE_NETWORK_TYPE = 6;
var ANDROID_X_DEVICE_IS_TYPE_CAPABLE = true;

function getLanguage() {
    return java.device.appUtils.getAppLanguage();
}

function getDateTimeFormat() {
    return ANDROID_X_BM_DATE_FORMAT;
}

function getTimeZoneCode() {
    var timezoneId = "056";
    if (isAndroid) {
        var microsoftTimezoneHelper = require("./MicrosoftTimezones.js");
        var timezoneName = java.device.calendar && java.device.calendar.getTimezoneName();
        var timezoneId = microsoftTimezoneHelper.getMSTimezoneID(timezoneName);
    }
    return parseInt(timezoneId);
}

function getProduct() {
    // todo: what is that?
    return ANDROID_X_DEVICE_PRODUCT;
}

function getManufacturer() {
    // todo: get from device
    return ANDROID_X_DEVICE_MANUFACTURER;
}

function getSKU() {
    // todo: what is that? it also doesn't appear in the request
    return ANDROID_X_DEVICE_SKU;
}

function getOsSKU() {
    // todo: what is that?
    return ANDROID_X_DEVICE_OS_SKU;
}

function getId() {
    // todo: what is that? where should we tak it from?
    return ANDROID_X_DEVICE_ID;
}

function getUserName() {
    return 'Ami';
}

function getAuthenticationTokens(cb) {

    if (isAndroid) {
        java.getAuthenticationTokens(function(err, authTokens) {
            return cb(err, authTokens);
        });
    }
    else return {
       mUID: 'muid',
       rpsToken: 'rpsToken',
       aisToken: 'aisToken'
    };
}

function getAisAuthentication(cb) {

    if (isAndroid) {
        java.getAuthenticationTokens(function(err, authTokens) {
            return cb(err, authTokens && authTokens.aisToken);
        });
    }
    else {
        return 'aisToken';
    };
}


function getRpsToken(cb) {

    if (isAndroid) {
        java.getAuthenticationTokens(function(err, authTokens) {
            return cb(err, authTokens && authTokens.rpsToken);
        });
    }
    else {
        return 'rpsToken';
    };
}

function getLocation() {
    if (isAndroid) {
        var location = java.device.location.getLastKnownLocation();
        if (location)
            return {
                latitude: location.getLatitude(),
                longitude: location.getLongitude(),
                time: location.getTime(),
                accuracy: location.getAccuracy()
            };
        else
            return null;
    }

    return {
       latitude: 1.222,
       longitude: 1.333,
       time: 111222333,
       accuracy: 5
    };
}

function getNetworkType() {
    // todo: what is that? where should we tak it from?
    return ANDROID_X_DEVICE_NETWORK_TYPE;
}

function getIsTouchCapable() {
    // todo: what is that? where should we tak it from?
    return ANDROID_X_DEVICE_IS_TYPE_CAPABLE;
}

function getHomeGeographicRegion() {
    return getLanguage().toLowerCase() == 'en-us' ? 'US' : 'CN';
}

function getRegistrationHeaders(cb) {
    headersHelper.getHeaders({endpoint:headersHelper.REGISTRATION_ID}, cb);
}

function getRemindersHeaders(cb) {
    headersHelper.getHeaders({endpoint:headersHelper.REMINDER_SERVICE}, cb);
}

function getEntityHeaders(cb) {
    headersHelper.getHeaders({endpoint:headersHelper.ENDPOINT_ENTITY}, cb);
}

function getProactiveHeaders(cb) {
    headersHelper.getHeaders({endpoint:headersHelper.SNR}, function(err, headers) {
        if(err) {
            console.error('error getting proactive headers:', err.toString());
            return cb(err);
        }
        console.log('got proactive headers:', headers);
        return cb(null, headers);
    });
}

function setBingCookies(cookies) {
    headersHelper.setBingCookies(cookies);
}

var api = {
    getProactiveHeaders: getProactiveHeaders,
    getLanguage: getLanguage,
    getHomeGeographicRegion: getHomeGeographicRegion,
    getRegistrationHeaders: getRegistrationHeaders,
    getRemindersHeaders: getRemindersHeaders,
    getEntityHeaders: getEntityHeaders,
    setBingCookies: setBingCookies
};

module.exports = api;


/*

    getDateTimeFormat: getDateTimeFormat,
    getTimeZoneCode: getTimeZoneCode,
    getProduct: getProduct,
    getManufacturer: getManufacturer,
    getSKU: getSKU,
    getOsSKU: getOsSKU,
    getId: getId,
    getAuthenticationTokens: getAuthenticationTokens,
    getAisAuthentication: getAisAuthentication,
    getLocation: getLocation,
    getNetworkType: getNetworkType,
    getIsTouchCapable: getIsTouchCapable,

    getUserName: getUserName
    */