/// <disable>JS2043.RemoveDebugCode</disable>
/// <disable>JS2064.SpecifyNewWhenCallingConstructor</disable>

var util = require('util');

// we use a translation table for oss modules to their dss correspondent modules
// some dss modules have the same interface as the oss ones, and some oss modules had to be renamed 
// in order to be packed inside an appx (ms).
require('modules-overwrite');

// Collect garbage to reduce memory consumption of un-needed memory
CollectGarbage();

// override console with the internal logger
function setConsoleAsLogger() {
  var logger = process.binding('logger_wrap').logger;

  var methods = {
    'debug': logger.logLevels.verbose,
    'log' : logger.logLevels.verbose , 
    'info' : logger.logLevels.info, 
    'warn' : logger.logLevels.warn, 
    'warning' : logger.logLevels.warn,  
    'error': logger.logLevels.error
  };

  for (var method in methods) {
    // whenever a console method will be called, we will call the logger with the appropriate log level
    console[method] = (function (methodName, i) {
      var logLevel = i;
      var originalMethod = console[methodName];
      
      // only turn on logging if there is a provider enabled.
      if (logger.isLogLevelEnabled(logLevel)) {        
        var logMethod = function () {
          logger.log(logLevel, util.format.apply(this, arguments));
          originalMethod.apply(this, arguments);
        }

        logMethod.isEnabled = true;
        return logMethod;
      }
      else {
        var logMethod = new Function();
        logMethod.isEnabled = false;
        return logMethod;
      }
    }).apply(this, [method, methods[method]]);
  }
};

setConsoleAsLogger();

if (process.argv.length < 3) {
  console.error('missing js file path parameter');
  process.exit(1);
}

var jsFilePath = process.argv[2];
require(jsFilePath);
