﻿// Copyright(c) Microsoft Corporation. All rights reserved.

var util = require('util');

// create a string from args trailing a function call, i.e. createError(logger, 'error: %s', e);
function formatTrailingArgs(start, callArgs) {
  var strArgs = [];
  
  for (var i = start; i < callArgs.length; i++) {
    strArgs.push(callArgs[i]);
  }
  
  return util.format.apply(undefined, strArgs);
}

// create an error with formatted string of args starting index [start] of callArgs.
function _createErrorInstance(start, callArgs) {
  return new Error(formatTrailingArgs(start, callArgs));
}

// Log an error object and add a tracker such that it won't be logged again.
function logError(logger, error) {
  if (!(error instanceof Error)) {
    return;
  }

  if (error.__dssLogged__) {
    return;
  }

  logger.error(error.message);
  logger.error(error.stack);
  error.__dssLogged__ = true;
}

// Call Cb with an error object after logging the error.
// optional parameters may an Error which will log only the error message.
//   i.e. return dssutil.cbError(cb, logger, err);
// or a string and formatters which will log the generated message and object.
//   i.e. return dssutil.cbError(cb, logger, 'invalid object: ', JSON.stringify(jsonObj));
function cbError(cb, logger) {
  var error = arguments[2];

  if (!(arguments[2] instanceof Error)) {
    error = _createErrorInstance(2, arguments);
  }
  
  logError(logger, error);
  return cb(error);
}

// Create an Error object and log the error and stack with parameters.
// e.g. dssutil.createError(logger, 'invalid object: ', JSON.stringify(jsonObj));
function createError(logger) {
  var error = _createErrorInstance(1, arguments);
  logError(logger, error);
  return error;
}

exports.formatTrailingArgs = formatTrailingArgs;
exports.createError = createError;
exports.cbError = cbError;
exports.logError = logError;