var utils = require('./utils.js');

function getEventsManager(java, javaCallbackWrapper) {

    if(!java) {
        console.info('not running in android... exiting eventsManager');
        return {
          start: function(){console.log('not running in android');},
          addListener: function(){console.log('not running in android');}
        };
    }

    var listeners = [];

    // register a handler for events coming from Java: function(event)
    function registerEventListener(cb) {

        // begin- temporarily disable java to node events
        // return console.warn('javaEventsManager registerEventListener\'s functionality is temporarily disabled (bug #1450586). we\'re working on fixing it.');
        // end- temporarily disable java to node events

        console.log('javaEventsManager.registerEventListener: threadId:', utils.getThreadId());
        var javaNodeEventEmitter = java.callStaticMethodSync('com.microsoft.bing.dss.nodelib.NodeEventEmitter', 'instance');
        var proxy = javaCallbackWrapper(cb, 'javaEventsManager\'s event listener');
        javaNodeEventEmitter.registerEventListener(proxy);
    }

    function start(cb) {
        registerEventListener(eventHandler);
        if(typeof cb === 'function') {
            addListener(cb);
        }
        console.log('listening for java events started. threadId:', utils.getThreadId());
    }

    function eventHandler(event) {

        var name = event.getEvent();
        var keys = event.getKeys();

        var eventObj = {
            name: event.getEvent(),
            context: event.getContext(),
            data: {}
        }

        for(var i=0; i<keys.length; i++) {
            var key = keys[i];
            eventObj.data[key] = event.get(key);
        }

        console.log('got event', eventObj, 'threadId:', utils.getThreadId());

        for(var i=0; i<listeners.length; i++)
            listeners[i](eventObj);
    }

    function addListener(handler) {
        if(typeof handler !== 'function') {
            throw new Error('handler should be a function');
        }
        console.log('javaEventsManager.addListener: threadId:', utils.getThreadId());
        listeners.push(handler)
    }

    var api = {
      start: start,
      addListener: addListener
    };

    return api;
}

module.exports = getEventsManager;