﻿/// <disable>JS2043.RemoveDebugCode</disable>
/// <disable>JS2087.DoNotDeleteObjectProperties</disable>

var path = require('path');
var request = require('request');
var fs = require('fs');
var async = require('async');
var mkdirp = require('mkdirp');
var settings = require('cortana.settings');
var tag = 'proactive/cache.js';

// Default paths
// TODO - Add to app settings


// TODO get
var appDataFolder = '/sdcard';

var cacheDirPath = path.join(appDataFolder, 'cache', 'proactive');
var basePageFilePath = path.join(cacheDirPath, 'basepage.html');
var answersFilePath = path.join(cacheDirPath, 'answers.json');
var timeStampFilePath = path.join(cacheDirPath, 'timestamp.txt');
var cacheEmptyError = new Error();
cacheEmptyError.name = 'CacheEmptyError';

var settingsContainer = new settings.SettingsContainer();

// TTL in milliseconds
var cacheTTL = 0;

// TODO: uncomment once the platform abstraction is finalized and cacheTTL can be read from container
// These lines are commented to disable reading cacheTTL from settings, until it's supported.
//try {
  //cacheTTL = settingsContainer.readDWORDValue('Proactive', 'CacheTTL');
//}
//catch (err) {
  // TODO Settings error, bail out
  //console.warn('%s: CacheTTL not specified - falling back to default (%d)', tag, cacheTTL, err);
//}

exports.setCachedAnswers = function setCachedAnswers (basePage, answers, cb) {
  
  mkdirp(cacheDirPath, function (err) {
    if (err) {
      return cb(err);
    }
    
    async.parallel([
      fs.writeFile.bind(this, basePageFilePath, basePage),
      fs.writeFile.bind(this, answersFilePath, JSON.stringify(answers, null, 2)),
      fs.writeFile.bind(this, timeStampFilePath, Date.now())
    ], cb);
  });
}

// Returns cached answers or 'CacheEmptyError'
exports.getCachedAnswers = function getCachedAnswers (basePageStream, cb) {
  
  fs.readFile(timeStampFilePath, function (err, timestamp) {
    if (err) {
      if (err.code === 'ENOENT') {
        // If timestamp file is missing, this is likely a first run so return a CacheEmptyError
        return cb(cacheEmptyError);
      }
      else {
        return cb(err);
      }
    }
    
    var cacheAge = Date.now() - timestamp;
    console.log('%s: Cache age: %d minutes.', tag, (cacheAge / 1000 / 60));
    console.log('%s: Cache TTL: %d minutes.', tag, (cacheTTL / 1000 / 60));

    // cacheAge can be < 0 only if the user changed the time manually,
    // in such case reload proactive (the new timestamp will be written from orchestrator.js).
    if (cacheAge < 0 || cacheAge > cacheTTL) {
      return cb(cacheEmptyError);
    }
    
    // Cache should exist and hasn't expired - read and return cached data
    async.parallel({
      basePageData : async.apply(fs.readFile, basePageFilePath), 
      answersData : async.apply(fs.readFile, answersFilePath)
    },
    function (err, results) {
      
      if (err) {
        cb(err);
      }
      
      try {
        var answers = JSON.parse(results.answersData);
      }
      catch (err) {
        return cb(err);
      }
      
      // Got both base page and answers - output both
      basePageStream.end(results.basePageData);
      return cb(null, answers)
 
    });
  });
}
