﻿/// <disable>JS2043.RemoveDebugCode</disable>
/// <disable>JS2087.DoNotDeleteObjectProperties</disable>
//npm modules
var url = require('url');
var util = require('util');
var request = require('request');
var multiparty = require('multiparty');
var java = require('java');
var async = require('async');
// cortana.*
var settings = require('cortana.settings');
// windows.*
var device = require('device');
//var requestHelpers = require('./request-helpers.js');

var SIMPLIFIED_ENGLISH_LANGUAGE_CODE = 'US';
var SIMPLIFIED_CHINESE_LANGUAGE_CODE = 'CN';

var tag = 'proactive/loader.js';
var settingsContainer = new settings.SettingsContainer();

// todo: take from phone
var languages = [device.getLanguage()];
var userProfile = {
    GlobalizationPreferences: {
        languages: languages,
        homeGeographicRegion: device.getHomeGeographicRegion()
    }
};

// get proactive endpoint from settings.
var ProactiveEndpoint = 'https://www.bing.com/proactive/v2';
try {
  ProactiveEndpoint = settingsContainer.readStringValue('Proactive', 'Endpoint');
}
catch (error) {
  console.warn('%s: Proactive endpoint not specified - falling back to default (%s)', tag, ProactiveEndpoint);
}

// get default request timeout
var requestTimeout = 5 * 1000; // 5 seconds
try {
  requestTimeout = settingsContainer.readDWORDValue('Proactive', 'RequestTimeout');
  console.log('%s: Request timeout value: %d ms', tag, requestTimeout);
}
catch (error) {
  console.warn('%s: Request timeout value not specified. Falling back to (%d) ms', tag, requestTimeout);
}

var SECTION_PAGE = 0;
var SECTION_ANSWERS = 1;

function refreshClientAnswers(clientExperienceProviders, cb) {
  console.log('%s: Fetching client answers', tag);
  // Call each of the client-side experience providers
  var funcs = {};

  // TODO:
  // Update provider.produce to accept a structured data object for options
  // instead of options string
  var options = '';
  for (var path in clientExperienceProviders) {
    funcs[path] = clientExperienceProviders[path].produce.bind(this, options);
  }

  async.parallel(funcs, function (err, results) {

    if (err) {
      return cb(err);
    }

    var clientAnswers = [];

    for (var path in results) {
      var result = results[path];
      if (result) {
        if (Array.isArray(result)) {
          clientAnswers = clientAnswers.concat(result);
        }
        else {
          clientAnswers.push(result);
        }
      }
    }

    cb(null, {
      Answers: clientAnswers
    });
  });
}

function constructProactiveUrl(params) {
  console.log('%s: Executing constructProactiveUrl(params)', tag);
  console.log('%s: params.appFormCode = %s , params.topID = %s', tag, params.appFormCode, params.topID);
  
  // holds the query string
  var searchQuery = [];

  if (params) {
    // append app code form if available
    if (params.appFormCode) {
      searchQuery.push(util.format('form=%s', params.appFormCode));
    }
    else {
      console.warn('%s: Missing appFormCode.', tag);
    }
	
	// append "topid" parameter if available
    if (params.topID) {
      searchQuery.push(util.format('topid=%s', params.topID));
    }
    else {
      console.log('%s: Missing topID.', tag);
    }
  }
  else {
    console.warn('%s: constructProactiveUrl: Missing params object.', tag);
  }

  var parsedProactiveEndpoint = url.parse(ProactiveEndpoint, true);

  // add fixed bing url
  searchQuery.push('addfeaturesnoexpansion=proactivejson');
  searchQuery.push('displaymode=full');

  // add language & geographical region
  if (languages && languages.length > 0) {
    parsedProactiveEndpoint.query['setlang'] = userProfile.GlobalizationPreferences.languages[0];
  }

  var ccParameterValue = java.device.appUtils.getAppLanguage().split('-')[1];
  console.log('%s: ccParameterValue = %s', tag, ccParameterValue);
  searchQuery.push(util.format('cc=%s', ccParameterValue));

  // append additional query parameters from settings if needed
  if (settingsContainer.valueExists('Proactive', 'ProactiveQueryParams')) {
    searchQuery.push(settingsContainer.readStringValue('Proactive', 'ProactiveQueryParams'));
  }

  // construct query string
  parsedProactiveEndpoint.search = searchQuery.join('&');
  console.log('parsedProactiveEndpoint', parsedProactiveEndpoint);
  return parsedProactiveEndpoint;
};

// returns true if internet connection is present, false otherwise.
function isConnectedToInternet() {
  return true;
  //TODO MSFT:XXX:windows.networking.connectivity fails under XDE
  /*
  var connectivity = require('windows.networking.connectivity');
  try {
    var connection = connectivity.NetworkInformation.getInternetConnectionProfile();
    // we only care about being connected to the internet..
    return connection && connection.getNetworkConnectivityLevel() == connectivity.NetworkConnectivityLevel.internetAccess;
  }
  catch (e) {
    console.warn('%s: Failed to query for current connection state. Error: %s', tag, e);
    return false;
  }*/
}

// fetches answers from bing.com/proactive
// params:
//  * viewDimension:  dimensions of the view port
//  * viewTheme:      current view port theme background vs accent colors)
//  * appFormCode:    [temp solution] holds the form code which was used to launch Cortana app
//  * topID:    	  holds the "topid" parameter which is added to proactive Url when user clicks on a notification
//  * userAgent:      [temp solution] TODO: make DSS able to create user-agent
// errors:
//  1) 'NoConnectionError' - the current client can't reach the internet.
//  2) 'RequestTimeoutError' - request to bing.com/proactive had timed out.
function refreshServerAnswers(basePageStream, params, cb) {
  console.log('%s: Fetching server answers', tag);

  // check if we have internet connection.
  if (!isConnectedToInternet()) {
    // we're not connected to the internet, bail out.
    var noConnectionError = new Error('No Internet Connection');
    noConnectionError.name = 'NoConnectionError';
    console.error('%s: No internet connection. Error: %s', tag, noConnectionError);

    return cb(noConnectionError);
  }

  var proactiveParsedUrl = constructProactiveUrl(params);

  //requestHelpers.getHeadersAndAddAuthCookies(params, proactiveParsedUrl, function (err, headers) {

  // inner callback function, the purpose of this callback is to make sure
  // that the outer callback is not called twice (calling the callback twice causes the async module to throw an exception)
  // the reason why the callback can be called multiple times as documented by multiparty node module:
  // "if you are listening to 'part' events, the same error may be emitted from the `form` and the `part`."
  // https://www.npmjs.com/package/multiparty
  var innerCallback = function(err, results) {
    console.log("refreshServerAnswer's inner callback is called");
    if(cb != null) {
        console.log("Delegating the inner callback call to the outer callback");
        cb(err, results);
        cb = null;
        return;
    }
    
    console.warn("innerCallback was called while the outer callback is null, request will not be delegated!");
  }
  
  device.getProactiveHeaders(function(err, headers) {
    if (err) {
      console.error('refreshServerAnswers', err);
      return innerCallback(err);
    }

    var uri = url.format(proactiveParsedUrl);
    if (java.device.appUtils.shouldShowAllAnswers()) {
        uri += '&debugScenario=All';
        console.log('%s: enable show all cards', tag);
    }

    var options = {
      uri: uri,
      headers: headers,
      timeout: requestTimeout
    };

    // call get
    console.log('device.getProactiveHeaders: options', options);
    var bingReq = request.get(options);
    bingReq.on('response', function (bingRes) {

      for(var header in bingRes.headers) {
        console.debug('**** got header:', header, ':', bingRes.headers[header]);
      }

      // Bing's FDv1 doesn't support returning the multipart header so we have to fix it up
      if (bingRes.headers['x-search-multipartboundary']) {
        bingRes.headers['content-type'] = 'multipart/form-data; boundary=' + bingRes.headers['x-search-multipartboundary'];
      }

      // Parse the multipart response
      var form = new multiparty.Form();

      var section = SECTION_PAGE;
      form.on('part', function (part) {

        if (section == SECTION_PAGE) {

          // Retrieve the base page and return it to the caller for rendering
          part.on('error', function (err) {
            console.warn('loader3');
            return innerCallback(err);
          });

          var contentType = part.headers['content-type'];
          part.headers = bingRes.headers;
          part.headers['content-type'] = contentType;
          // pipe result out to base page stream

          var partChunks = [];
          part.on('data', function (chunk) {
            partChunks.push(chunk);
          });

          part.on('end', function () {
              var basePage = Buffer.concat(partChunks);
              basePageStream.end(basePage);
          });

          //part.pipe(basePageStream);
          // The answers section is next
          section = SECTION_ANSWERS;
        }
        else {
          // Parse the server-generated answers and cache them
          var chunks = [];

          part.on('data', function (chunk) {
            chunks.push(chunk);
          });

          part.on('end', function () {
            try {
              var serverAnswers = JSON.parse(Buffer.concat(chunks));
            }
            catch (err) {
              return innerCallback(err);
            }
            return innerCallback(null, serverAnswers);
          });

          part.on('error', function (err) {
            return innerCallback(err);
          });
        }
      });

      form.on('error', function (err) {
        return innerCallback(err);
      });

      form.parse(bingRes);
    });

    bingReq.on('error', function (err) {
      if (err && err.code == 'ETIMEDOUT') {
        var requestTimeoutError = new Error('Request to server had timed out.');
        requestTimeoutError.name = 'RequestTimeoutError';
        console.error('%s: Request to %s had timed out. Error: %s', tag, proactiveParsedUrl.url, err);

        return innerCallback(requestTimeoutError);
      }
      return innerCallback(err);
    });
  });
}

// fetches answers from both bing.com/proactive & client experiences
// params:
//  * viewDimension:  dimensions of the view port
//  * viewTheme:      current view port theme background vs accent colors)
//  * appFormCode:    [temp solution] holds the form code which was used to launch Cortana app
//  * userAgent:      [temp solution] TODO: make DSS able to create user-agent
exports.fetchAllAnswers = function (params, basePageStream, clientExperienceProviders, cb) {
  async.parallel(
    {
      serverResults: async.apply(refreshServerAnswers, basePageStream, params),
      clientResults: async.apply(refreshClientAnswers, clientExperienceProviders)
    }, function fetchCompleted(err, results) {

      if (err) {
        return cb(err);
      }

      if (!results || !results.serverResults || !results.clientResults) {
        return cb(new Error('Fetch operation failed'));
      }

      var basePageGlobals = {};
      var serverAnswers = [];
      var clientAnswers = [];

      if (results.serverResults.BasePage) {
        basePageGlobals = results.serverResults.BasePage;
      }
      else {
        console.warn('%s: missing BasePage section', tag);
      }

      if (Array.isArray(results.serverResults.Answers)) {
        serverAnswers = results.serverResults.Answers;
      }
      else {
        console.warn('%s: missing Answers in server response', tag);
      }

      if (Array.isArray(results.clientResults.Answers)) {
        clientAnswers = results.clientResults.Answers;
      }
      else {
        console.warn('%s: missing Answers in client response', tag);
      }

      var result = {
                     BasePage: basePageGlobals,
                     Answers: serverAnswers.concat(clientAnswers)
                   };

      return cb(null, result);
    }
  );
};