/// <disable>JS2043.RemoveDebugCode</disable>
/// <disable>JS2087.DoNotDeleteObjectProperties</disable>

// npm modules
var async = require('async');
var util = require('util');
var request = require('request');
var device = require('device');


// defines the prefix for a valid MUID
var muidCookiePrefix = 'MUID=';

// todo: take from phone
var languages = [device.getLanguage()];

var systemHelpers = {
    regionalSettings: device.getLanguage(),
    dateTimeFormat: device.getDateTimeFormat(),
    timeZone: device.getTimeZoneCode(),
    product: device.getProduct(),
    manufacturer: device.getManufacturer(),
    deviceSKU: device.getSKU(),
    operatingSystemSKU: device.getOsSKU(),
    deviceId: device.getId()
};

var tag = 'proactive/request-helpers';


setCookiesIfNotExist = function(headers, cookies) {
    console.log('setCookiesIfNotExist');

    if(!headers['cookie'])
        headers['cookie'] = '';

    for (var i=0; i<cookies.length; i++) {
        if (headers['cookie'])
            headers['cookie'] += '; ';
        headers['cookie'] += cookies[i];
    }
}


// Gets MUID using cortana.Authentication APIs
// returns -> { mUID: ''}
//  
function getAuthenticationTokens(cb) {
    return device.getAuthenticationTokens(cb);
};

// Calls into Cortana.Authentication and attempts to refresh the AIS token 
function getAISToken(cb) {
    return device.getAisAuthentication(cb);
};

function getRPSToken(cb) {
    return device.getRpsToken(cb);
};

// This calls into Cortana.Location APIs to get the actual location header value to avoid 
// duplicating some of the location formatting code which is used by other Cortana components
// TODO: 
// Investigate using WinRT location APIs directly 
function getLocationHeaderValue(cb) {

  // todo: which format? didn't see this is the fiddler request
  var location = device.getLocation();
  if (location) {
    var locationString = 'lat:' + location.latitude
                       + ';long:' + location.longitude
                       + ';ts:' + parseInt(location.time/1000)
                       + ';re:' + location.accuracy;
    return cb(null, locationString);
  }
  else {
      return cb(new Error("device location is unavailable"));
  }

  /*
  var location = require('cortana.location');
  var locationMgr = new location.LocationManager();
  // TODO: 
  // Move location settinsg (timeout, accuracy, warmup..) to settings. 
  locationMgr.getLocationHeaderAsync('Cortana_Proactive', 800, 1800, 0, true, function (err, result) {
    if (err) {
      console.error('%s: getLocationHeaderAsync Error', tag, err);
      return cb(err);
    }
    return cb(null, result);
  });
  */
};

// gets time in epoch format
function getEpochTime() {
  return (new Date()).getTime();
};

// gets the current device touch capability 
function isTouchCapable() {
  return device.getIsTouchCapable();
};

// queries the current network connectivity.  
// returns { ianaType: number, dataClass: as wwanDataClass }  

function getNetworkConnectivity() {
  var connectivity = require('windows.networking.connectivity');
  var connection = connectivity.NetworkInformation.getInternetConnectionProfile();

  var ianaType = 0;
  var wwanDataClass = connectivity.WwanDataClass.none;

  if (connection) {
    ianaType = connection.networkAdapter.ianaInterfaceType;

    if (connection.isWwanConnectionProfile) {
      var wwlanConnection = connection.wwanConnectionProfileDetails;
      wwanDataClass = wwlanConnection.getCurrentDataClass();
    }
  }

  return { ianaType: ianaType, dataClass: wwanDataClass };
};

// gets the list of the supported application languages
function getAppLanguages() {
  return languages;
};

// gets X-Device-* and X-BM-* headers
function getDeviceRequestHeaders(params) {

  var headers = {};

  // check if we have any query parameters 
  // coming from the calling view port
  if (params) {
    // add headers from query params. 
    if (params.viewDimension) {
      headers['x-bm-devicedimensions'] = params.viewDimension;
    }

    if (params.viewTheme) {
      headers['x-bm-theme'] = params.viewTheme;
    }

    if (params.userAgent) {
      // add the user-agent header 
      headers['user-agent'] = params.userAgent;
    }
  }
  else {
    console.warn('%s: getRequestHeadersSync: No Query Params were recieved.', tag);
  }

  (function addAppLanguagesHeader() {
    // add app languages
    var languages = getAppLanguages();
    var languesHeaderValues = '';
    if (languages && languages.length > 0) {
      languesHeaderValues = languages[0];

      for (var i = 1; i < languages.length; i++) {
        languesHeaderValues.concat(util.format(', %s', languages[i]));
      };

      headers['Accept-language'] = languesHeaderValues;
    }
  })();



  (function addXbmHeaders() {
    console.log('%s: Adding X-BM headers.', tag);

    // epoch time
    var result = getEpochTime();
    if (result) {
      headers['X-BM-CBT'] = result.toString();
    }

    try {
      headers['X-BM-RegionalSettings'] = systemHelpers.regionalSettings;
    }
    catch (err) {
      console.warn('%s: Failed to get regional settings. Error: %s', tag, err);
    }

    try {
      headers['X-BM-DateFormat'] = systemHelpers.dateTimeFormat;
    }
    catch (err) {
      console.warn('%s: Failed to get dateTimeFormat. Error: %s', tag, err);
    }

    try {
      headers['X-BM-DTZ'] = systemHelpers.timeZone;
    }
    catch (err) {
      console.warn('%s: Failed to get timeZone. Error: %s', tag, err);
    }

    try {
      headers['X-BM-UserDisplayName'] = device.getUserName();
    }
    catch (err) {
      console.warn('%s: Failed to get user name. Error: %s', tag, err);
    }





  })();

  (function addXDeviceHeaders() {
    console.log('%s: Adding device headers.', tag);

    try {
      headers['X-Device-Product'] = systemHelpers.product;
    }
    catch (err) {
      console.warn('%s: Failed to get Product. Error: %s', tag, err.toString());
    }

    try {
      headers['X-Device-Manufacturer'] = systemHelpers.manufacturer;
    }
    catch (err) {
      console.warn('%s: Failed to get Manufacturer. Error: %s', tag, err);
    }

    try {
      headers['X-Device-SKU'] = systemHelpers.deviceSKU;
    }
    catch (err) {
      console.warn('%s: Failed to get Device SKU. Error: %s', tag, err);
    }

    try {
      headers['X-Device-OSSKU'] = systemHelpers.operatingSystemSKU;
    }
    catch (err) {
      console.warn('%s: Failed to get OS SKU. Error: %s', tag, err);
    }

    try {
      headers['X-Device-MachineId'] = systemHelpers.deviceId;
    }
    catch (err) {
      console.warn('%s: Failed to get MachineId. Error: %s', tag, err);
    }

    headers['X-Device-Touch'] = isTouchCapable().toString();
    headers['X-P2P-PeerDist'] = 'Version=1.1';
    headers['X-P2P-PeerDistEx'] = 'MinContentInformation=1.0, MaxContentInformation=2.0';

    /*
    try {
      var connection = getNetworkConnectivity();
      var connectionHeaderValue;
      if (connection) {
        if (connection.dataClass) {
          // this is a cellular connection.  
          connectionHeaderValue = util.format('%d-%d', connection.ianaType, connection.dataClass);
        }
        else {
          connectionHeaderValue = util.format('%d', connection.ianaType);
        }
      }

      headers['X-Device-NetworkType'] = connectionHeaderValue;
    }
    catch (e) {
      console.warn('%s: Failed to add Device-NetworkType header. Error: %s', tag, e);
    }

    */
    headers['X-Device-NetworkType'] = device.getNetworkType();

  })();

  return headers;
}

// adds authentication cookies to request. 
// cookies will be added to all requests made from the hosting proc. 
function addAuthCookies(proactiveUrl, headers, cb) {
  // get the value of auth cookies  
  getAuthenticationTokens(function (err, results) {
    if (err) {
      // we bail out. 
      console.error('%s: Failed to get authentication tokens. Erorr: ', tag, err);
      return cb(err);
    }

    if (results) {
      try {
        // set protocol to https
        proactiveUrl.protocol = 'https';
        // set MUID

        var cookies = [];
        cookies.push(muidCookiePrefix + results.muid);

        setCookiesIfNotExist(headers, cookies);

        return cb();
      }
      catch (e) {
        // can't add cookies. Bail out.
        console.error('%s: failed to add cookie:', e);

        return cb(e);
      }
    }
  });
};

// gets and appends required headers to fetch bing.com/proactive
exports.getHeadersAndAddAuthCookies = function getHeadersAndAddAuthCookies(params, parsedProactiveUrl, cb) {

  // get bing headers 
  var headers = getDeviceRequestHeaders(params);
  async.parallel({
    location: getLocationHeaderValue.bind(this),
    aisToken: getAISToken.bind(this),
    rpsToken: getRpsToken.bind(this),
    authCookies: addAuthCookies.bind(this, parsedProactiveUrl, headers)
  },
      function (err, results) {
        if (err) {
          return cb(err);
        }
        // set location 
        headers['x-search-location'] = results.location;
        // set ais-token
        headers['x-ais-authtoken'] = results.aisToken;
        // set rpsToken
        headers['x-search-rpstoken'] = results.rpsToken;
        // set host 
        headers['Host'] = parsedProactiveUrl.hostname;

        return cb(null, headers);
      });
};