var middler = require('middler');
var domain = require('domain');
var url = require('url');
var proxy = require('./proxy.js');
var settings = require('cortana.settings');
var settingsContainer = new settings.SettingsContainer();
var ProactiveExperienceService = require('proactive').ProactiveExperienceService;
var collector = null;
var app = router = middler();

var tag = 'proactive/index.js';

try {
  if (settingsContainer.readBoolValue('Proactive', 'CallDebuggerEnabledInDssStartupCode')) {
    debugger;
  }
}
catch (err) {
  console.warn('%s: CallDebuggerEnabledInDssStartupCode not specified - ignoring', tag, err);
}


// Add domain support for the incoming request.
// This promises that exceptions that happen in the context of a specific request will be logged and treated in the right context.
router.first('*', function(req, res, next) {
  var d = domain.create();
  d.pathname = url.parse(req.url).pathname;
  
  d.on('error', function(e) {
    try {
      console.error('%s: Error serving: %s', tag, req.url, e);
      if(e.stack) {
        console.error('%s: Stack trace: %', tag, e.stack);
      }
      res.writeHead(500, { 'content-type': 'text/plain' });
      return res.end('Internal server error.');
    }
    catch(error) {
       console.error('%s: Error sending 500!', tag, error.stack);
    }
  });

  // Because req and res were created before this domain existed,
  // we need to explicitly add them.
  // See the explanation of implicit vs explicit binding below.
  d.add(req);
  d.add(res);

  // Move function in the domain.
  return d.run(function() {
    return next();
  });
});

router.first('/devicecontent/*', function (req, res, next) {
  req.url = req.url.replace('/devicecontent', '');
  next();
});

// Start client experience providers
// TODO - Revisit the connection logic between client experiences and proactive, e.g.:
//    a) how does a new provider register itself?
//    b) how do we communicate the registered providers to the proactive loader?
//    c) how to lazily load providers when they are required
var providers = require('client-providers');
providers.start(router);

// Create the proactive services instance
var proactive = new ProactiveExperienceService(providers.clientExperienceProviders);

// Start answers service
console.log('%s: starting proactive answers service', tag);
require('answers').start(router, proactive);

// Proxies all remaining requests to https://www.bing.com
console.log('%s: adding proxy to router', tag);
proxy.addToRouter(router);

router.last(function(req, res) {
  console.log('%s: [%d] 404 FILE NOT FOUND', tag, req.__reqId);
  res.writeHead(404, { 'Content-Type': 'text/plain' });
  return res.end('Not found.');
});

console.log('%s: DSS ready for incoming requests', tag);

module.exports = app;
