var url = require('url');
var request = require('request');
var java = require('java');
var tag = 'proactive/proxy.js';
var settings = require('cortana.settings');
var settingsContainer = new settings.SettingsContainer();
var device = require('device');

function stringifyHeaders(headers) {
  var headersString;
  Object.keys(headers).forEach(function (key) {
        headersString += key + ': ' + headers[key] + '  ';
  });
  return headersString;
}

var proactiveHeaders;
var ProactiveEndpoint = 'https://www.bing.com/proactive/v2';
try {
  ProactiveEndpoint = settingsContainer.readStringValue('Proactive', 'Endpoint');
  console.log('%s: proactive endpoint is %s', tag, ProactiveEndpoint);
}
catch (error) {
  console.warn('%s: Proactive endpoint not specified - falling back to default (%s)', tag, ProactiveEndpoint);
}

var proxyRequest = function proxyRequest (req, res, next) {
  delete req.headers.host;

  var parsedUrl = url.parse(req.url);
  console.log('%s: [%d] proxying request to server', tag, req.__reqId, req.url);

  parsedUrl.host = 'www.bing.com';
  try {
    parsedUrl.host = settingsContainer.readStringValue('Proactive', 'Host');
  }
  catch (error) {
    console.warn('%s: Host is not specified - falling back to default (%s)', tag, parsedUrl.host);
  }
  parsedUrl.protocol = 'https';

 var requestHeaders = {};
 
 if (proactiveHeaders) {
     Object.keys(req.headers).forEach(function (key) {
       requestHeaders[key.toLowerCase()] = req.headers[key];
     });
     Object.keys(proactiveHeaders).forEach(function (key) {
         requestHeaders[key.toLowerCase()] = proactiveHeaders[key];
     });
   } else {
     console.log('%s: No proactiveHeaders were cached', tag);
     requestHeaders = req.headers;
   }

  var options = {
    uri: url.format(parsedUrl),
    method: req.method,
    headers: requestHeaders,
    useCache: true,
  };

  if (options.headers[request.COOKIE_HEADER]) {
    try {
      console.log('%s: [%d] setting incoming cookies in proxy cookie jar: %s', tag, req.__reqId, options.headers[request.COOKIE_HEADER]);
      request.setCookiesIfNotExist(parsedUrl, options.headers[request.COOKIE_HEADER]);
    }
    catch (e) {
      console.error('%s: [%d] failed on request.setCookiesIfNotExist, returning 500, error: %s', tag, req.__reqId, e);
      res.writeHead(500, { 'content-type': 'text/plain' });
      return res.end('Internal server error.');
    }
    delete options.headers[request.COOKIE_HEADER];
  }

 console.log('%s: Dest server:%s body:%s', tag, options.uri, req.body);

  var bingReq = request(options);

  bingReq.on('error', function (err) {
    console.error('%s: [%d] proxy request failed, [%s] error: %s', tag, req.__reqId, req.url, err);
    next();
  });

  bingReq.on('response', function (res) {
    if (res.headers) {
      var responseHeaders = 'HTTP/1.1 ' + res.statusCode + '  ' + stringifyHeaders(res.headers);

      console.log('%s: [%d] server response headers: %s', tag, req.__reqId, responseHeaders);

      // Set the cookies received from bing.
      if (res.headers['set-cookie']) {
        device.setBingCookies(java.createJavaArray('java.lang.String', res.headers['set-cookie']));
      }
    }
  });

  req.pipe(bingReq).pipe(res);
}

exports.addToRouter = function (router) {
  console.log('%s: adding all remaining pages to proxy route', tag);

  // Proxy all remaining requests to https://www.bing.com || https://cn.bing.com  We might eventually want
  // to make this a little more selective.
  router.add('*', function (req, res, next) {
    if (!proactiveHeaders) {
      console.log('%s: calling getProactiveHeaders', tag);
      device.getProactiveHeaders(function (err, headers) {
        if (err) {
          console.error('%s: getProactiveHeaders failed, err: %s', tag, err.toString());
          res.writeHead(500, { 'content-type': 'text/plain' });
          return res.end('Internal server error.');
        }
        proactiveHeaders = headers;
        proxyRequest(req, res, next);
      });
    } else {
      return proxyRequest(req, res, next);
    }
  });
};