﻿
var types = require('./types.js');
var timeHandler = new (require('./time.js').TimeHandler)();
var locationHandler = new (require('./location.js').LocationHandler)();


function TypeToHandler() {
  if (!this instanceof TypeToHandler) {
    return new TypeToHandler();
  }
  
  // Map of the reminder type to the handler which understands this type.
  this._handlers = {};
  this._handlers[types.ReminderType.Time] = timeHandler;
  this._handlers[types.ReminderType.Location] = locationHandler;
  // In TH all location reminders are TimeLocation reminders.
  this._handlers[types.ReminderType.TimeLocation] = locationHandler;
  
  this.typeToHandler = function (type) {
    return this._handlers[type] || this._errorHandler;
  };
  
  this._errorCallback = function () {
    return arguments[arguments.length - 1]('invalid type');
  };
  
  // Dummy handler for unsupported types.
  this._errorHandler = {
    parseFromV1 : function () { return false; },
    parseToV1 : function () { return false; },
    deleteReminder: this._errorCallback,
    updateReminder: this._errorCallback,
    createReminder: this._errorCallback
  };
}

var _typeToHandler = new TypeToHandler();
exports.typeToHandler = _typeToHandler.typeToHandler.bind(_typeToHandler);
