﻿// Copyright(c) Microsoft Corporation. All rights reserved.
var java = require('java');
var location = java.device.locationApi;

var LOCATION_NAMESPACE = 'com.microsoft.bing.dss.platform.location.';
var GEOFENCE_NAMESPACE = LOCATION_NAMESPACE + 'geofence.';
var GEOFENCE_DESCRIPTOR_TYPE = GEOFENCE_NAMESPACE + 'GeofenceDescriptor';
var GEOFENCE_CALLBACK_TYPE = GEOFENCE_NAMESPACE + 'GeofenceCallback';

function OnCompleteWrapper(cb) {
  if (!(this instanceof OnCompleteWrapper)) {
    return new OnCompleteWrapper(cb);
  }
  
  cb = cb || Function();
  
  this.__cb = cb;
  
  this.onComplete = function (err, result) {
    if (err) {
      return this.__cb(err.toString());
    }
    
    return this.__cb(null, result);
  };
  
  return this;
}

// handler is called with a GeofenceDescriptor when a geofence is entered.
function onEnteringGeofence(handler) {
  return location.on('entering_geofence', handler);
}

// handler is called with a GeofenceDescriptor when a geofence is exited.
function onExittingGeofence(handler) {
  return location.on('exiting_geofence', handler);
}

// register persistently for geofence transitions according to the descriptor.  cb is called with no results.
function registerGeofence(geofenceDescriptor, cb) {
  var callback = java.createJavaProxy(GEOFENCE_CALLBACK_TYPE, new OnCompleteWrapper(cb));

  return location.registerGeofence(geofenceDescriptor, callback);
}

// unregister a previously registered geofence specified by the id. cb is called with no results.
function unregisterGeofence(geofenceId, cb) {
  var callback = java.createJavaProxy(GEOFENCE_CALLBACK_TYPE, new OnCompleteWrapper(cb));

  return location.unregisterGeofence(geofenceId, callback);
}

// create an instance of GeofenceDescriptor.
function createGeofenceDescriptor(tag, extraData, latitude, longitude, radius, expireTime, transition) {
  return java.createJavaInstance(GEOFENCE_DESCRIPTOR_TYPE,
                                 tag,
                                 extraData,
                                 latitude,
                                 longitude,
                                 Math.round(radius), // sometimes many decimal places... don't need that precision.
                                 expireTime,
                                 transition,
                                 0); // lingering not supported.
}

// Constants for accessing the location API.
function Constants() {
  if (!(this instanceof Constants)) {
    return new Constants();
  }

  this.NEVER_EXPIRE = -1;
  this.GEOFENCE_TRANSITION_ENTER = 0x1;
  this.GEOFENCE_TRANSITION_EXIT = 0x2;
}

exports.constants = new Constants();
exports.onEnteringGeofence = onEnteringGeofence;
exports.onExittingGeofence = onExittingGeofence;
exports.registerGeofence = registerGeofence;
exports.unregisterGeofence = unregisterGeofence;
exports.createGeofenceDescriptor = createGeofenceDescriptor;