﻿// Copyright(c) Microsoft Corporation. All rights reserved.
var java = require('java');
var reminders = java.device.reminder;

var REMINDER_NAMESPACE = 'com.microsoft.bing.dss.platform.reminderSync.';
var REMINDER_CALLBACK_TYPE = REMINDER_NAMESPACE + 'ReminderCallback';
var REMINDER_QUERYONE_CALLBACK_TYPE = REMINDER_NAMESPACE + 'ReminderQueryOneCallback';
var REMINDER_QUERYMULTIPLE_CALLBACK_TYPE = REMINDER_NAMESPACE + 'ReminderQueryMultipleCallback';
var REMINDER_QUERYMULTIPLE_IDS_CALLBACK_TYPE = REMINDER_NAMESPACE + 'ReminderQueryMultipleIdsCallback';
var REMINDER_DESCRIPTOR_TYPE = REMINDER_NAMESPACE + 'ReminderDescriptor';

// Wrapper for all completions, accepts the completion routine and forwards the result on javascript completion.
function OnCompleteWrapper(cb) {
  if (!(this instanceof OnCompleteWrapper)) {
    return new OnCompleteWrapper(cb);
  }

  cb = cb || Function();

  this.__cb = cb;

  this.onComplete = function (err, result) {
    if (err) {
      return this.__cb(err.toString());
    }
    
    return this.__cb(null, result);
  };

  return this;
}

// queries for all reminders with the sync status (true | false) and completes with a ReminderDescriptor[];
function queryBySyncStatus(status, cb) {
  var callback = java.createJavaProxy(REMINDER_QUERYMULTIPLE_CALLBACK_TYPE, new OnCompleteWrapper(cb));

  return reminders.queryBySyncStatus(status, callback);
}

// deletes the reminder with the matching local id. completes with no results.
function deleteByLocalId(id, cb) {
  var callback = java.createJavaProxy(REMINDER_CALLBACK_TYPE, new OnCompleteWrapper(cb));
  
  return reminders.deleteByLocalId(id, callback);
}

// queries the reminder with the matching local id.  completes with a single ReminderDescriptor.
function queryByLocalId(id, cb) {
  var callback = java.createJavaProxy(REMINDER_QUERYONE_CALLBACK_TYPE, new OnCompleteWrapper(cb));
  
  return reminders.queryByLocalId(id, callback);
}

// queries the reminder with the matching server id.  completes with a single ReminderDescriptor.
function queryByServerId(id, cb) {
  var callback = java.createJavaProxy(REMINDER_QUERYONE_CALLBACK_TYPE, new OnCompleteWrapper(cb));
  
  return reminders.queryByServerId(id, callback);
}

// queries all reminders. completes with a ReminderDescriptor[]
function queryAll(cb) {
  var callback = java.createJavaProxy(REMINDER_QUERYMULTIPLE_CALLBACK_TYPE, new OnCompleteWrapper(cb));
  
  return reminders.queryAll(callback);
}

// creates or updates the corresponding reminderDescriptor in the Db. Completes with no results.
function createOrUpdateReminder(reminderDescriptor, cb) {
  var callback = java.createJavaProxy(REMINDER_CALLBACK_TYPE, new OnCompleteWrapper(cb));
  
  return reminders.createOrUpdateReminder(reminderDescriptor, callback);
}

// queries all entries in the database that are not in the given list of server IDs.  completes with a ReminderDescriptor[]
function queryWhereServerIdNot(ids, cb) {
  var javaIds = java.createJavaArray('java.lang.String', ids);

  var callback = java.createJavaProxy(REMINDER_QUERYMULTIPLE_IDS_CALLBACK_TYPE, new OnCompleteWrapper(cb));
  
  return reminders.queryWhereServerIdNot(javaIds, callback);
}

// triggers a reminder on the device.
function triggerReminder(localId) {
  return reminders.triggerReminder(localId);
}

// create an instance of ReminderDescriptor.
function createReminderDescriptor() {
  return java.createJavaInstance(REMINDER_DESCRIPTOR_TYPE);
}

function notifyRemindersUpdated() {
  return reminders.notifyRemindersUpdated();
}

function clearReminderDb(cb) {
    var callback = java.createJavaProxy(REMINDER_CALLBACK_TYPE, new OnCompleteWrapper(cb));
    return reminders.deleteAllReminders(callback);
}

exports.queryBySyncStatus = queryBySyncStatus;
exports.deleteByLocalId = deleteByLocalId;
exports.queryByLocalId = queryByLocalId;
exports.queryByServerId = queryByServerId;
exports.queryAll = queryAll;
exports.createOrUpdateReminder = createOrUpdateReminder;
exports.queryWhereServerIdNot = queryWhereServerIdNot;
exports.triggerReminder = triggerReminder;
exports.createReminderDescriptor = createReminderDescriptor;
exports.notifyReminderUpdate = notifyRemindersUpdated;
exports.clearLocalReminderDb = clearReminderDb;