﻿// Copyright(c) Microsoft Corporation. All rights reserved.

var java = require('java');
var timers = java.device.timers;

var ALARM_NAMESPACE = 'com.microsoft.bing.dss.platform.alarms.';
var ALARM_CALLBACK_TYPE = ALARM_NAMESPACE + 'AlarmCallback';
var ALARM_DESCRIPTOR_TYPE = ALARM_NAMESPACE + 'AlarmDescriptor';

// Wrapper for all completions, accepts the completion routine and forwards the result on javascript completion.
function OnCompleteWrapper(cb) {
  if (!(this instanceof OnCompleteWrapper)) {
    return new OnCompleteWrapper(cb);
  }
  
  cb = cb || Function();
  
  this.__cb = cb;
  
  this.onComplete = function (err, result) {
    if (err) {
      return this.__cb(err.toString());
    }
    
    return this.__cb(null, result);
  };
  
  return this;
}

// Create an instance of AlarmDescriptor.
function createTimerDescriptor(timerType, id, startTime, interval, extraData) {
  return java.createJavaInstance(ALARM_DESCRIPTOR_TYPE,
                                 timerType,
                                 id,
                                 java.createJavaInstance('java.util.Date', new Date(startTime).toString()),
                                 java.createJavaInstance('java.lang.Long', interval.toString()),
                                 extraData);
}

// Create a persistent timer that will fire according to the specified interval.  Completes when persisted, no result.
function createPersistentTimer(timerdescriptor, cb) {
  var callback = java.createJavaProxy(ALARM_CALLBACK_TYPE, new OnCompleteWrapper(cb));

  return timers.setPersistentAlarm(timerdescriptor, callback);
}

// Cancel a persistent timer.  Completes when cancelled, no result.
function cancelPersistentTimer(id, cb) {
  var callback = java.createJavaProxy(ALARM_CALLBACK_TYPE, new OnCompleteWrapper(cb));

  return timers.cancelPersistentAlarm(id, callback);
}

// Subscribe to be notified when timers fire.  handler will receive an AlarmDescriptor.
function onTimer(handler) {
  return timers.on('persistent_alarm', handler);
}

// The types of timers supported.
function TimerType() {
  if (!(this instanceof TimerType)) {
    return new TimerType();
  }

  // Fire repeating, but no need to be exact.
  this.inexact_repeating = timers.getAlarmTypeEnum('INEXACT_REPEATING');

  // Fire only once at the time specified.
  this.one_time = timers.getAlarmTypeEnum('ONE_TIME');
  
  return this;
}

exports.timerType = new TimerType();
exports.createTimerDescriptor = createTimerDescriptor;
exports.createPersistentTimer = createPersistentTimer;
exports.cancelPersistentTimer = cancelPersistentTimer;
exports.onTimer = onTimer;