// Copyright(c) Microsoft Corporation. All rights reserved.

var types = require('./types.js');
var platform_location = require('./platform-location.js');

// The types for timeReminder.recurrence
var remindersRecurrenceV1 = {
  None    : 'None',
  Daily   : 'Daily',
  Weekly  : 'Weekly'
};

// Maps from V2 occurrence to supported V2 types.
var reminderRecurrenceV2toV1Map = {};
reminderRecurrenceV2toV1Map[types.ReminderFrequency.AsItHappens]   =  remindersRecurrenceV1.None;
reminderRecurrenceV2toV1Map[types.ReminderFrequency.Weekly]        =  remindersRecurrenceV1.Weekly;
reminderRecurrenceV2toV1Map[types.ReminderFrequency.Daily]         =  remindersRecurrenceV1.Daily;
reminderRecurrenceV2toV1Map[types.ReminderFrequency.Hourly]        =  remindersRecurrenceV1.Hourly;
reminderRecurrenceV2toV1Map[types.ReminderFrequency.TwiceADay]     =  remindersRecurrenceV1.Daily;
reminderRecurrenceV2toV1Map[types.ReminderFrequency.FourTimesADay] =  remindersRecurrenceV1.Daily;
reminderRecurrenceV2toV1Map[types.ReminderFrequency.Biweekly]      =  remindersRecurrenceV1.Weekly;
reminderRecurrenceV2toV1Map[types.ReminderFrequency.Once]          =  remindersRecurrenceV1.None;

// Maps from V1 recurrence to supported V2 types.
var reminderRecurrenceV1toV2Map = {};
reminderRecurrenceV1toV2Map[remindersRecurrenceV1.None] = 'Once';
reminderRecurrenceV1toV2Map[remindersRecurrenceV1.Daily] = 'Daily';
reminderRecurrenceV1toV2Map[remindersRecurrenceV1.Weekly] = 'Weekly';


// Maps from named V2 status to numbered V1 status.
var reminderStatusV2toV1Map = {};
reminderStatusV2toV1Map[types.ReminderStatus.Inactive] = 0;
reminderStatusV2toV1Map[types.ReminderStatus.Active] = 1;
reminderStatusV2toV1Map[types.ReminderStatus.Snoozed] = 2;
reminderStatusV2toV1Map[types.ReminderStatus.Cancelled] = 3;
reminderStatusV2toV1Map[types.ReminderStatus.Completed] = 4;

//Maps from numbered V1 status to named V2 status.
var reminderStatusV1toV2Map = {};
reminderStatusV1toV2Map[0] = types.ReminderStatus.Inactive;
reminderStatusV1toV2Map[1] = types.ReminderStatus.Active;
reminderStatusV1toV2Map[2] = types.ReminderStatus.Snoozed;
reminderStatusV1toV2Map[3] = types.ReminderStatus.Cancelled;
reminderStatusV1toV2Map[4] = types.ReminderStatus.Completed;

var geofenceKindV2toV1Map = {};
//Not sure why it would be none - but it will not work on android.
geofenceKindV2toV1Map[types.ReminderGeoFenceType.None] = platform_location.constants.GEOFENCE_TRANSITION_ENTER;
geofenceKindV2toV1Map[types.ReminderGeoFenceType.Arrive] = platform_location.constants.GEOFENCE_TRANSITION_ENTER;
geofenceKindV2toV1Map[types.ReminderGeoFenceType.Depart] = platform_location.constants.GEOFENCE_TRANSITION_EXIT;
//We don't have nearby.
geofenceKindV2toV1Map[types.ReminderGeoFenceType.NearBy] = platform_location.constants.GEOFENCE_TRANSITION_ENTER;

var geofenceKindV1ToV2 = {};
geofenceKindV1ToV2[platform_location.constants.GEOFENCE_TRANSITION_ENTER] = types.ReminderGeoFenceType.Arrive;
geofenceKindV1ToV2[platform_location.constants.GEOFENCE_TRANSITION_EXIT] = types.ReminderGeoFenceType.Depart;


function validate(v1Obj) {
  // Copy to the V2 manifest.
  if (v1Obj.title === undefined ||
      v1Obj.detail === undefined ||
      v1Obj.status === undefined ||
      reminderStatusV1toV2Map[v1Obj.status] === undefined) {
    return false;
  }
  
  return true;
}

exports.ReminderStatusV2toV1Map = reminderStatusV2toV1Map;
exports.RemindersRecurrenceV1 = remindersRecurrenceV1;
exports.ReminderStatusV1toV2Map = reminderStatusV1toV2Map;
exports.ReminderRecurrenceV2toV1Map = reminderRecurrenceV2toV1Map;
exports.ReminderRecurrenceV1toV2Map = reminderRecurrenceV1toV2Map;
exports.GeofenceKindV2toV1Map = geofenceKindV2toV1Map;
exports.GeofenceKindV1ToV2 = geofenceKindV1ToV2;
exports.validate = validate;